# This file has been autogenerated by the pywayland scanner

# Copyright © 2014, 2015 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlBuffer
from .wayland import WlBufferProxy
from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class ZwpLinuxBufferParamsV1(Interface):
    """Parameters for creating a dmabuf-based :class:`~pywayland.protocol.wayland.WlBuffer`

    This temporary object is a collection of dmabufs and other parameters that
    together form a single logical buffer. The temporary object may eventually
    create one :class:`~pywayland.protocol.wayland.WlBuffer` unless cancelled
    by destroying it before requesting 'create'.

    Single-planar formats only require one dmabuf, however multi-planar formats
    may require more than one dmabuf. For all formats, an 'add' request must be
    called once per plane (even if the underlying dmabuf fd is identical).

    You must use consecutive plane indices ('plane_idx' argument for 'add')
    from zero to the number of planes used by the drm_fourcc format code. All
    planes required by the format must be given exactly once, but can be given
    in any order. Each plane index can only be set once; subsequent calls with
    a plane index which has already been set will result in a plane_set error
    being generated.
    """

    name = "zwp_linux_buffer_params_v1"
    version = 6

    class error(enum.IntEnum):
        already_used = 0
        plane_idx = 1
        plane_set = 2
        incomplete = 3
        invalid_format = 4
        invalid_dimensions = 5
        out_of_bounds = 6
        invalid_wl_buffer = 7
        invalid_dev_t_size = 8

    class flags(enum.IntFlag):
        y_invert = 1
        interlaced = 2
        bottom_first = 4


class ZwpLinuxDmabufFeedbackV1(Interface):
    """Dmabuf feedback

    This object advertises dmabuf parameters feedback. This includes the
    preferred devices and the supported formats/modifiers.

    The parameters are sent once when this object is created and whenever they
    change. The done event is always sent once after all parameters have been
    sent. When a single parameter changes, all parameters are re-sent by the
    compositor.

    Compositors can re-send the parameters when the current client buffer
    allocations are sub-optimal. Compositors should not re-send the parameters
    if re-allocating the buffers would not result in a more optimal
    configuration. In particular, compositors should avoid sending the exact
    same parameters multiple times in a row.

    The tranche_target_device and tranche_formats events are grouped by
    tranches of preference. For each tranche, a tranche_target_device, one
    tranche_flags and one or more tranche_formats events are sent, followed by
    a tranche_done event finishing the list. The tranches are sent in
    descending order of preference. All formats and modifiers in the same
    tranche have the same preference.

    To send parameters, the compositor sends one main_device event (unless the
    client bound version 6 or above), tranches (each consisting of one
    tranche_target_device event, one tranche_flags event, tranche_formats
    events and then a tranche_done event), then one done event.

    With version 6 and above, the compositor must always advertise at least one
    tranche with the sampling flag set.
    """

    name = "zwp_linux_dmabuf_feedback_v1"
    version = 6

    class tranche_flags(enum.IntFlag):
        scanout = 1
        sampling = 2


class ZwpLinuxDmabufV1(Interface):
    """Factory for creating dmabuf-based wl_buffers

    This interface offers ways to create generic dmabuf-based wl_buffers.

    For more information about dmabuf, see:
    https://www.kernel.org/doc/html/next/userspace-api/dma-buf-alloc-
    exchange.html

    Clients can use the get_surface_feedback request to get dmabuf feedback for
    a particular surface. If the client wants to retrieve feedback not tied to
    a surface, they can use the get_default_feedback request.

    The following are required from clients:

    - Clients must ensure that either all data in the dma-buf is coherent for
      all subsequent read access or that coherency is correctly handled by the
      underlying kernel-side dma-buf implementation.

    - Don't make any more attachments after sending the buffer to the
      compositor. Making more attachments later increases the risk of the
      compositor not being able to use (re-import) an existing dmabuf-based
      :class:`~pywayland.protocol.wayland.WlBuffer`.

    The underlying graphics stack must ensure the following:

    - The dmabuf file descriptors relayed to the server will stay valid for the
      whole lifetime of the :class:`~pywayland.protocol.wayland.WlBuffer`. This
      means the server may at any time use those fds to import the dmabuf into
      any kernel sub-system that might accept it.

    However, when the underlying graphics stack fails to deliver the promise,
    because of e.g. a device hot-unplug which raises internal errors, after the
    :class:`~pywayland.protocol.wayland.WlBuffer` has been successfully created
    the compositor must not raise protocol errors to the client when dmabuf
    import later fails.

    To create a :class:`~pywayland.protocol.wayland.WlBuffer` from one or more
    dmabufs, a client creates a
    :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1` object
    with a :func:`ZwpLinuxDmabufV1.create_params()` request. All planes
    required by the intended format are added with the 'add' request. Finally,
    a 'create' or 'create_immed' request is issued, which has the following
    outcome depending on the import success.

    The 'create' request, - on success, triggers a 'created' event which
    provides the final   :class:`~pywayland.protocol.wayland.WlBuffer` to the
    client. - on failure, triggers a 'failed' event to convey that the server
    cannot use the dmabufs received from the client.

    For the 'create_immed' request, - on success, the server immediately
    imports the added dmabufs to   create a
    :class:`~pywayland.protocol.wayland.WlBuffer`. No event is sent from the
    server in this case. - on failure, the server can choose to either:   -
    terminate the client by raising a fatal error.   - mark the
    :class:`~pywayland.protocol.wayland.WlBuffer` as failed, and send a
    'failed' event to the     client. If the client uses a failed
    :class:`~pywayland.protocol.wayland.WlBuffer` as an argument to any
    request, the behaviour is compositor implementation-defined.

    For all DRM formats and unless specified in another protocol extension,
    pre-multiplied alpha is used for pixel values.

    Unless specified otherwise in another protocol extension, implicit
    synchronization is used. In other words, compositors and clients must wait
    and signal fences implicitly passed via the DMA-BUF's reservation
    mechanism.
    """

    name = "zwp_linux_dmabuf_v1"
    version = 6


class ZwpLinuxBufferParamsV1Resource(Resource[ZwpLinuxBufferParamsV1]):
    interface = ZwpLinuxBufferParamsV1

    @ZwpLinuxBufferParamsV1.event(
        Argument(ArgumentType.NewId, interface=WlBuffer),
    )
    def created(self, buffer: WlBuffer) -> None:
        """Buffer creation succeeded

        This event indicates that the attempted buffer creation was successful.
        It provides the new :class:`~pywayland.protocol.wayland.WlBuffer`
        referencing the dmabuf(s).

        Upon receiving this event, the client should destroy the
        :class:`ZwpLinuxBufferParamsV1` object.

        :param buffer:
            id for the the newly created
            :class:`~pywayland.protocol.wayland.WlBuffer`
        :type buffer:
            :class:`~pywayland.protocol.wayland.WlBuffer`
        """
        self._post_event(0, buffer)

    @ZwpLinuxBufferParamsV1.event()
    def failed(self) -> None:
        """Buffer creation failed

        This event indicates that the attempted buffer creation has failed. It
        usually means that one of the dmabuf constraints has not been
        fulfilled.

        Upon receiving this event, the client should destroy the
        :class:`ZwpLinuxBufferParamsV1` object.
        """
        self._post_event(1)


class ZwpLinuxDmabufFeedbackV1Resource(Resource[ZwpLinuxDmabufFeedbackV1]):
    interface = ZwpLinuxDmabufFeedbackV1

    @ZwpLinuxDmabufFeedbackV1.event()
    def done(self) -> None:
        """All feedback has been sent

        This event is sent after all parameters of a
        :class:`ZwpLinuxDmabufFeedbackV1` object have been sent.

        This allows changes to the :class:`ZwpLinuxDmabufFeedbackV1` parameters
        to be seen as atomic, even if they happen via multiple events.
        """
        self._post_event(0)

    @ZwpLinuxDmabufFeedbackV1.event(
        Argument(ArgumentType.FileDescriptor),
        Argument(ArgumentType.Uint),
    )
    def format_table(self, fd: int, size: int) -> None:
        """Format and modifier table

        This event provides a file descriptor which can be memory-mapped to
        access the format and modifier table.

        The table contains a tightly packed array of consecutive format +
        modifier pairs. Each pair is 16 bytes wide. It contains a format as a
        32-bit unsigned integer, followed by 4 bytes of unused padding, and a
        modifier as a 64-bit unsigned integer. The native endianness is used.

        The client must map the file descriptor in read-only private mode.

        Compositors are not allowed to mutate the table file contents once this
        event has been sent. Instead, compositors must create a new, separate
        table file and re-send feedback parameters. Compositors are allowed to
        store duplicate format + modifier pairs in the table.

        :param fd:
            table file descriptor
        :type fd:
            `ArgumentType.FileDescriptor`
        :param size:
            table size, in bytes
        :type size:
            `ArgumentType.Uint`
        """
        self._post_event(1, fd, size)

    @ZwpLinuxDmabufFeedbackV1.event(
        Argument(ArgumentType.Array),
    )
    def main_device(self, device: list) -> None:
        """Preferred main device

        This event advertises the main device that the server prefers to use
        when direct scan-out to the target device isn't possible. The
        advertised main device may be different for each
        :class:`ZwpLinuxDmabufFeedbackV1` object, and may change over time.

        There is exactly one main device. The compositor must send at least one
        preference tranche with tranche_target_device equal to main_device.

        Clients need to create buffers that the main device can import and read
        from, otherwise creating the dmabuf
        :class:`~pywayland.protocol.wayland.WlBuffer` will fail (see the
        :func:`ZwpLinuxBufferParamsV1.create()
        <pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1.create>` and
        create_immed requests for details). The main device will also likely be
        kept active by the compositor, so clients can use it instead of waking
        up another device for power savings.

        In general the device is a DRM node. The DRM node type (primary vs.
        render) is unspecified. Clients must not rely on the compositor sending
        a particular node type. Clients cannot check two devices for equality
        by comparing the dev_t value.

        If explicit modifiers are not supported and the client performs buffer
        allocations on a different device than the main device, then the client
        must force the buffer to have a linear layout.

        With version 6 and above, this event is no longer sent. Clients should
        use a device with the sampling flag in the tranches instead.

        :param device:
            device dev_t value
        :type device:
            `ArgumentType.Array`
        """
        self._post_event(2, device)

    @ZwpLinuxDmabufFeedbackV1.event()
    def tranche_done(self) -> None:
        """A preference tranche has been sent

        This event splits tranche_target_device and tranche_formats events into
        preference tranches. It is sent after a set of tranche_target_device
        and tranche_formats events; it represents the end of a tranche. The
        next tranche will have a lower preference.
        """
        self._post_event(3)

    @ZwpLinuxDmabufFeedbackV1.event(
        Argument(ArgumentType.Array),
    )
    def tranche_target_device(self, device: list) -> None:
        """Target device

        This event advertises the target device that the server prefers to use
        for a buffer created given this tranche. The advertised target device
        may be different for each preference tranche, and may change over time.

        There is exactly one target device per tranche.

        The target device may be a scan-out device, for example if the
        compositor prefers to directly scan-out a buffer created given this
        tranche. The target device may be a rendering device, for example if
        the compositor prefers to texture from said buffer.

        The client can use this hint to allocate the buffer in a way that makes
        it accessible from the target device, ideally directly. The buffer must
        still be accessible from a device with the sampling flag, either
        through direct import or a potentially more expensive fallback path. If
        the buffer can't be directly imported for sampling, then clients must
        be prepared for the compositor changing the tranche priority or making
        :class:`~pywayland.protocol.wayland.WlBuffer` creation fail (see the
        :func:`ZwpLinuxBufferParamsV1.create()
        <pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1.create>` and
        create_immed requests for details).

        If the device is a DRM node, the DRM node type (primary vs. render) is
        unspecified. Clients must not rely on the compositor sending a
        particular node type. Clients cannot check two devices for equality by
        comparing the dev_t value.

        This event is tied to a preference tranche, see the tranche_done event.

        :param device:
            device dev_t value
        :type device:
            `ArgumentType.Array`
        """
        self._post_event(4, device)

    @ZwpLinuxDmabufFeedbackV1.event(
        Argument(ArgumentType.Array),
    )
    def tranche_formats(self, indices: list) -> None:
        """Supported buffer format modifiers

        This event advertises the format + modifier combinations that the
        compositor supports.

        It carries an array of indices, each referring to a format + modifier
        pair in the last received format table (see the format_table event).
        Each index is a 16-bit unsigned integer in native endianness.

        For legacy support, DRM_FORMAT_MOD_INVALID is an allowed modifier. It
        indicates that the server can support the format with an implicit
        modifier. When a buffer has DRM_FORMAT_MOD_INVALID as its modifier, it
        is as if no explicit modifier is specified. The effective modifier will
        be derived from the dmabuf.

        A compositor that sends valid modifiers and DRM_FORMAT_MOD_INVALID for
        a given format supports both explicit modifiers and implicit modifiers.

        Compositors must not send duplicate format + modifier pairs within the
        same tranche or across two different tranches with the same target
        device and flags.

        This event is tied to a preference tranche, see the tranche_done event.

        For the definition of the format and modifier codes, see the
        :func:`ZwpLinuxBufferParamsV1.create()
        <pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1.create>`
        request.

        :param indices:
            array of 16-bit indexes
        :type indices:
            `ArgumentType.Array`
        """
        self._post_event(5, indices)

    @ZwpLinuxDmabufFeedbackV1.event(
        Argument(ArgumentType.Uint),
    )
    def tranche_flags(self, flags: int) -> None:
        """Tranche flags

        This event sets tranche-specific flags. This event is tied to a
        preference tranche, see the tranche_done event. With version 6 and
        above, the compositor must set at least one flag in each tranche.

        :param flags:
            tranche flags
        :type flags:
            `ArgumentType.Uint`
        """
        self._post_event(6, flags)


class ZwpLinuxDmabufV1Resource(Resource[ZwpLinuxDmabufV1]):
    interface = ZwpLinuxDmabufV1

    @ZwpLinuxDmabufV1.event(
        Argument(ArgumentType.Uint),
    )
    def format(self, format: int) -> None:
        """Supported buffer format

        This event advertises one buffer format that the server supports. All
        the supported formats are advertised once when the client binds to this
        interface. A roundtrip after binding guarantees that the client has
        received all supported formats.

        For the definition of the format codes, see the
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1`::create
        request.

        Starting version 4, the format event is deprecated and must not be sent
        by compositors. Instead, use get_default_feedback or
        get_surface_feedback.

        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        """
        self._post_event(0, format)

    @ZwpLinuxDmabufV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        version=3,
    )
    def modifier(self, format: int, modifier_hi: int, modifier_lo: int) -> None:
        """Supported buffer format modifier

        This event advertises the formats that the server supports, along with
        the modifiers supported for each format. All the supported modifiers
        for all the supported formats are advertised once when the client binds
        to this interface. A roundtrip after binding guarantees that the client
        has received all supported format-modifier pairs.

        For legacy support, DRM_FORMAT_MOD_INVALID (that is, modifier_hi ==
        0x00ffffff and modifier_lo == 0xffffffff) is allowed in this event. It
        indicates that the server can support the format with an implicit
        modifier. When a plane has DRM_FORMAT_MOD_INVALID as its modifier, it
        is as if no explicit modifier is specified. The effective modifier will
        be derived from the dmabuf.

        A compositor that sends valid modifiers and DRM_FORMAT_MOD_INVALID for
        a given format supports both explicit modifiers and implicit modifiers.

        For the definition of the format and modifier codes, see the
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1`::create
        and
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1`::add
        requests.

        Starting version 4, the modifier event is deprecated and must not be
        sent by compositors. Instead, use get_default_feedback or
        get_surface_feedback.

        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        :param modifier_hi:
            high 32 bits of layout modifier
        :type modifier_hi:
            `ArgumentType.Uint`
        :param modifier_lo:
            low 32 bits of layout modifier
        :type modifier_lo:
            `ArgumentType.Uint`
        """
        self._post_event(1, format, modifier_hi, modifier_lo)


class ZwpLinuxBufferParamsV1Proxy(Proxy[ZwpLinuxBufferParamsV1]):
    interface = ZwpLinuxBufferParamsV1

    @ZwpLinuxBufferParamsV1.request()
    def destroy(self) -> None:
        """Delete this object, used or not

        Cleans up the temporary data sent to the server for dmabuf-based
        :class:`~pywayland.protocol.wayland.WlBuffer` creation.
        """
        self._marshal(0)
        self._destroy()

    @ZwpLinuxBufferParamsV1.request(
        Argument(ArgumentType.FileDescriptor),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def add(self, fd: int, plane_idx: int, offset: int, stride: int, modifier_hi: int, modifier_lo: int) -> None:
        """Add a dmabuf to the temporary set

        This request adds one dmabuf to the set in this
        :class:`ZwpLinuxBufferParamsV1`.

        The 64-bit unsigned value combined from modifier_hi and modifier_lo is
        the dmabuf layout modifier. DRM AddFB2 ioctl calls this the fb
        modifier, which is defined in drm_mode.h of Linux UAPI. This is an
        opaque token. Drivers use this token to express tiling, compression,
        etc. driver-specific modifications to the base format defined by the
        DRM fourcc code.

        Starting from version 4, the invalid_format protocol error is sent if
        the format + modifier pair was not advertised as supported.

        Starting from version 5, the invalid_format protocol error is sent if
        all planes don't use the same modifier.

        This request raises the PLANE_IDX error if plane_idx is too large. The
        error PLANE_SET is raised if attempting to set a plane that was already
        set.

        :param fd:
            dmabuf fd
        :type fd:
            `ArgumentType.FileDescriptor`
        :param plane_idx:
            plane index
        :type plane_idx:
            `ArgumentType.Uint`
        :param offset:
            offset in bytes
        :type offset:
            `ArgumentType.Uint`
        :param stride:
            stride in bytes
        :type stride:
            `ArgumentType.Uint`
        :param modifier_hi:
            high 32 bits of layout modifier
        :type modifier_hi:
            `ArgumentType.Uint`
        :param modifier_lo:
            low 32 bits of layout modifier
        :type modifier_lo:
            `ArgumentType.Uint`
        """
        self._marshal(1, fd, plane_idx, offset, stride, modifier_hi, modifier_lo)

    @ZwpLinuxBufferParamsV1.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def create(self, width: int, height: int, format: int, flags: int) -> None:
        """Create a :class:`~pywayland.protocol.wayland.WlBuffer` from the given dmabufs

        This asks for creation of a
        :class:`~pywayland.protocol.wayland.WlBuffer` from the added dmabuf
        buffers. The :class:`~pywayland.protocol.wayland.WlBuffer` is not
        created immediately but returned via the 'created' event if the dmabuf
        sharing succeeds. The sharing may fail at runtime for reasons a client
        cannot predict, in which case the 'failed' event is triggered.

        The 'format' argument is a DRM_FORMAT code, as defined by the libdrm's
        drm_fourcc.h. The Linux kernel's DRM sub-system is the authoritative
        source on how the format codes should work.

        The 'flags' is a bitfield of the flags defined in enum "flags".
        'y_invert' means that the image needs to be y-flipped.

        Flag 'interlaced' means that the frame in the buffer is not progressive
        as usual, but interlaced. An interlaced buffer as supported here must
        always contain both top and bottom fields. The top field always begins
        on the first pixel row. The temporal ordering between the two fields is
        top field first, unless 'bottom_first' is specified. It is undefined
        whether 'bottom_first' is ignored if 'interlaced' is not set.

        This protocol does not convey any information about field rate,
        duration, or timing, other than the relative ordering between the two
        fields in one buffer. A compositor may have to estimate the intended
        field rate from the incoming buffer rate. It is undefined whether the
        time of receiving :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` with a new buffer
        attached, applying the :class:`~pywayland.protocol.wayland.WlSurface`
        state, :func:`WlSurface.frame()
        <pywayland.protocol.wayland.WlSurface.frame>` callback trigger,
        presentation, or any other point in the compositor cycle is used to
        measure the frame or field times. There is no support for detecting
        missed or late frames/fields/buffers either, and there is no support
        whatsoever for cooperating with interlaced compositor output.

        The composited image quality resulting from the use of interlaced
        buffers is explicitly undefined. A compositor may use elaborate
        hardware features or software to deinterlace and create progressive
        output frames from a sequence of interlaced input buffers, or it may
        produce substandard image quality. However, compositors that cannot
        guarantee reasonable image quality in all cases are recommended to just
        reject all interlaced buffers.

        Any argument errors, including non-positive width or height, mismatch
        between the number of planes and the format, bad format, bad offset or
        stride, may be indicated by fatal protocol errors: INCOMPLETE,
        INVALID_FORMAT, INVALID_DIMENSIONS, OUT_OF_BOUNDS.

        Dmabuf import errors in the server that are not obvious client bugs are
        returned via the 'failed' event as non-fatal. This allows attempting
        dmabuf sharing and falling back in the client if it fails.

        This request can be sent only once in the object's lifetime, after
        which the only legal request is destroy. This object should be
        destroyed after issuing a 'create' request. Attempting to use this
        object after issuing 'create' raises the ALREADY_USED protocol error.

        It is not mandatory to issue 'create'. If a client wants to cancel the
        buffer creation, it can just destroy this object.

        :param width:
            base plane width in pixels
        :type width:
            `ArgumentType.Int`
        :param height:
            base plane height in pixels
        :type height:
            `ArgumentType.Int`
        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        :param flags:
            see enum flags
        :type flags:
            `ArgumentType.Uint`
        """
        self._marshal(2, width, height, format, flags)

    @ZwpLinuxBufferParamsV1.request(
        Argument(ArgumentType.NewId, interface=WlBuffer),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        version=2,
    )
    def create_immed(self, width: int, height: int, format: int, flags: int) -> WlBufferProxy:
        """Immediately create a :class:`~pywayland.protocol.wayland.WlBuffer` from the given                      dmabufs

        This asks for immediate creation of a
        :class:`~pywayland.protocol.wayland.WlBuffer` by importing the added
        dmabufs.

        In case of import success, no event is sent from the server, and the
        :class:`~pywayland.protocol.wayland.WlBuffer` is ready to be used by
        the client.

        Upon import failure, either of the following may happen, as seen fit by
        the implementation: - the client is terminated with one of the
        following fatal protocol   errors:   - INCOMPLETE, INVALID_FORMAT,
        INVALID_DIMENSIONS, OUT_OF_BOUNDS,     in case of argument errors such
        as mismatch between the number     of planes and the format, bad
        format, non-positive width or     height, or bad offset or stride.   -
        INVALID_WL_BUFFER, in case the cause for failure is unknown or
        platform specific. - the server creates an invalid
        :class:`~pywayland.protocol.wayland.WlBuffer`, marks it as failed and
        sends a 'failed' event to the client. The result of using this
        invalid :class:`~pywayland.protocol.wayland.WlBuffer` as an argument in
        any request by the client is   defined by the compositor
        implementation.

        This takes the same arguments as a 'create' request, and obeys the same
        restrictions.

        :param width:
            base plane width in pixels
        :type width:
            `ArgumentType.Int`
        :param height:
            base plane height in pixels
        :type height:
            `ArgumentType.Int`
        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        :param flags:
            see enum flags
        :type flags:
            `ArgumentType.Uint`
        :returns:
            :class:`~pywayland.protocol.wayland.WlBuffer` -- id for the newly
            created :class:`~pywayland.protocol.wayland.WlBuffer`
        """
        buffer_id = self._marshal_constructor(3, WlBuffer, width, height, format, flags)
        return buffer_id

    @ZwpLinuxBufferParamsV1.request(
        Argument(ArgumentType.Array),
        version=6,
    )
    def set_sampling_device(self, device: list) -> None:
        """Set the target device of the :class:`~pywayland.protocol.wayland.WlBuffer`

        Set the device the compositor should import the dmabufs to for sampling
        in the next create or create_immed request.

        To avoid race conditions when the compositor removes a device from the
        tranches, it is not a protocol error if the device hasn't been
        advertised by the compositor in a tranche with the sampling flag, but
        the import is likely to fail in that case.

        If the client doesn't know a suitable target device, it shouldn't set
        one, and the compositor should attempt import on all devices it
        supports.

        If the array is too small to contain a dev_t or larger than required,
        the invalid_dev_t_size error will be emitted.

        :param device:
            device dev_t value
        :type device:
            `ArgumentType.Array`
        """
        self._marshal(4, device)


class ZwpLinuxDmabufFeedbackV1Proxy(Proxy[ZwpLinuxDmabufFeedbackV1]):
    interface = ZwpLinuxDmabufFeedbackV1

    @ZwpLinuxDmabufFeedbackV1.request()
    def destroy(self) -> None:
        """Destroy the feedback object

        Using this request a client can tell the server that it is not going to
        use the :class:`ZwpLinuxDmabufFeedbackV1` object anymore.
        """
        self._marshal(0)
        self._destroy()


class ZwpLinuxDmabufV1Proxy(Proxy[ZwpLinuxDmabufV1]):
    interface = ZwpLinuxDmabufV1

    @ZwpLinuxDmabufV1.request()
    def destroy(self) -> None:
        """Unbind the factory

        Objects created through this interface, especially wl_buffers, will
        remain valid.
        """
        self._marshal(0)
        self._destroy()

    @ZwpLinuxDmabufV1.request(
        Argument(ArgumentType.NewId, interface=ZwpLinuxBufferParamsV1),
    )
    def create_params(self) -> ZwpLinuxBufferParamsV1Proxy:
        """Create a temporary object for buffer parameters

        This temporary object is used to collect multiple dmabuf handles into a
        single batch to create a :class:`~pywayland.protocol.wayland.WlBuffer`.
        It can only be used once and should be destroyed after a 'created' or
        'failed' event has been received.

        :returns:
            :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1`
            -- id for the newly created
            :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxBufferParamsV1`
        """
        params_id = self._marshal_constructor(1, ZwpLinuxBufferParamsV1)
        return params_id

    @ZwpLinuxDmabufV1.request(
        Argument(ArgumentType.NewId, interface=ZwpLinuxDmabufFeedbackV1),
        version=4,
    )
    def get_default_feedback(self) -> ZwpLinuxDmabufFeedbackV1Proxy:
        """Get default feedback

        This request creates a new
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxDmabufFeedbackV1`
        object not bound to a particular surface. This object will deliver
        feedback about dmabuf parameters to use if the client doesn't support
        per-surface feedback (see get_surface_feedback).

        :returns:
            :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxDmabufFeedbackV1`
        """
        id = self._marshal_constructor(2, ZwpLinuxDmabufFeedbackV1)
        return id

    @ZwpLinuxDmabufV1.request(
        Argument(ArgumentType.NewId, interface=ZwpLinuxDmabufFeedbackV1),
        Argument(ArgumentType.Object, interface=WlSurface),
        version=4,
    )
    def get_surface_feedback(self, surface: WlSurfaceProxy) -> ZwpLinuxDmabufFeedbackV1Proxy:
        """Get feedback for a surface

        This request creates a new
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxDmabufFeedbackV1`
        object for the specified
        :class:`~pywayland.protocol.wayland.WlSurface`. This object will
        deliver feedback about dmabuf parameters to use for buffers attached to
        this surface.

        If the surface is destroyed before the
        :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxDmabufFeedbackV1`
        object, the feedback object becomes inert.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.linux_dmabuf_v1.ZwpLinuxDmabufFeedbackV1`
        """
        id = self._marshal_constructor(3, ZwpLinuxDmabufFeedbackV1, surface)
        return id


class ZwpLinuxBufferParamsV1Global(Global[ZwpLinuxBufferParamsV1]):
    interface = ZwpLinuxBufferParamsV1


class ZwpLinuxDmabufFeedbackV1Global(Global[ZwpLinuxDmabufFeedbackV1]):
    interface = ZwpLinuxDmabufFeedbackV1


class ZwpLinuxDmabufV1Global(Global[ZwpLinuxDmabufV1]):
    interface = ZwpLinuxDmabufV1


ZwpLinuxBufferParamsV1._gen_c()
ZwpLinuxBufferParamsV1.proxy_class = ZwpLinuxBufferParamsV1Proxy
ZwpLinuxBufferParamsV1.resource_class = ZwpLinuxBufferParamsV1Resource
ZwpLinuxBufferParamsV1.global_class = ZwpLinuxBufferParamsV1Global


ZwpLinuxDmabufFeedbackV1._gen_c()
ZwpLinuxDmabufFeedbackV1.proxy_class = ZwpLinuxDmabufFeedbackV1Proxy
ZwpLinuxDmabufFeedbackV1.resource_class = ZwpLinuxDmabufFeedbackV1Resource
ZwpLinuxDmabufFeedbackV1.global_class = ZwpLinuxDmabufFeedbackV1Global


ZwpLinuxDmabufV1._gen_c()
ZwpLinuxDmabufV1.proxy_class = ZwpLinuxDmabufV1Proxy
ZwpLinuxDmabufV1.resource_class = ZwpLinuxDmabufV1Resource
ZwpLinuxDmabufV1.global_class = ZwpLinuxDmabufV1Global
