# This file has been autogenerated by the pywayland scanner

# Copyright © 2024 Xaver Hugl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class WpAlphaModifierSurfaceV1(Interface):
    """Alpha modifier object for a surface

    This interface allows the client to set a factor for the alpha values on a
    surface, which can be used to offload such operations to the compositor.
    The default factor is UINT32_MAX.

    This object has to be destroyed before the associated
    :class:`~pywayland.protocol.wayland.WlSurface`. Once the
    :class:`~pywayland.protocol.wayland.WlSurface` is destroyed, all request on
    this object will raise the no_surface error.
    """

    name = "wp_alpha_modifier_surface_v1"
    version = 1

    class error(enum.IntEnum):
        no_surface = 0


class WpAlphaModifierV1(Interface):
    """Surface alpha modifier manager

    This interface allows a client to set a factor for the alpha values on a
    surface, which can be used to offload such operations to the compositor,
    which can in turn for example offload them to KMS.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "wp_alpha_modifier_v1"
    version = 1

    class error(enum.IntEnum):
        already_constructed = 0


class WpAlphaModifierSurfaceV1Resource(Resource[WpAlphaModifierSurfaceV1]):
    interface = WpAlphaModifierSurfaceV1


class WpAlphaModifierV1Resource(Resource[WpAlphaModifierV1]):
    interface = WpAlphaModifierV1


class WpAlphaModifierSurfaceV1Proxy(Proxy[WpAlphaModifierSurfaceV1]):
    interface = WpAlphaModifierSurfaceV1

    @WpAlphaModifierSurfaceV1.request()
    def destroy(self) -> None:
        """Destroy the alpha modifier object

        This destroys the object, and is equivalent to set_multiplier with a
        value of UINT32_MAX, with the same double-buffered semantics as
        set_multiplier.
        """
        self._marshal(0)
        self._destroy()

    @WpAlphaModifierSurfaceV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_multiplier(self, factor: int) -> None:
        """Specify the alpha multiplier

        Sets the alpha multiplier for the surface. The alpha multiplier is
        double-buffered state, see :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` for details.

        This factor is applied in the compositor's blending space, as an
        additional step after the processing of per-pixel alpha values for the
        :class:`~pywayland.protocol.wayland.WlSurface`. The exact meaning of
        the factor is thus undefined, unless the blending space is specified in
        a different extension.

        This multiplier is applied even if the buffer attached to the
        :class:`~pywayland.protocol.wayland.WlSurface` doesn't have an alpha
        channel; in that case an alpha value of one is used instead.

        Zero means completely transparent, UINT32_MAX means completely opaque.

        :param factor:
        :type factor:
            `ArgumentType.Uint`
        """
        self._marshal(1, factor)


class WpAlphaModifierV1Proxy(Proxy[WpAlphaModifierV1]):
    interface = WpAlphaModifierV1

    @WpAlphaModifierV1.request()
    def destroy(self) -> None:
        """Destroy the alpha modifier manager object

        Destroy the alpha modifier manager. This doesn't destroy objects
        created with the manager.
        """
        self._marshal(0)
        self._destroy()

    @WpAlphaModifierV1.request(
        Argument(ArgumentType.NewId, interface=WpAlphaModifierSurfaceV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_surface(self, surface: WlSurfaceProxy) -> WpAlphaModifierSurfaceV1Proxy:
        """Create a new alpha modifier surface object

        Create a new alpha modifier surface object associated with the given
        :class:`~pywayland.protocol.wayland.WlSurface`. If there is already
        such an object associated with the
        :class:`~pywayland.protocol.wayland.WlSurface`, the already_constructed
        error will be raised.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.alpha_modifier_v1.WpAlphaModifierSurfaceV1`
        """
        id = self._marshal_constructor(1, WpAlphaModifierSurfaceV1, surface)
        return id


class WpAlphaModifierSurfaceV1Global(Global[WpAlphaModifierSurfaceV1]):
    interface = WpAlphaModifierSurfaceV1


class WpAlphaModifierV1Global(Global[WpAlphaModifierV1]):
    interface = WpAlphaModifierV1


WpAlphaModifierSurfaceV1._gen_c()
WpAlphaModifierSurfaceV1.proxy_class = WpAlphaModifierSurfaceV1Proxy
WpAlphaModifierSurfaceV1.resource_class = WpAlphaModifierSurfaceV1Resource
WpAlphaModifierSurfaceV1.global_class = WpAlphaModifierSurfaceV1Global


WpAlphaModifierV1._gen_c()
WpAlphaModifierV1.proxy_class = WpAlphaModifierV1Proxy
WpAlphaModifierV1.resource_class = WpAlphaModifierV1Resource
WpAlphaModifierV1.global_class = WpAlphaModifierV1Global
