# This file has been autogenerated by the pywayland scanner

# Copyright © 2015-2016 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class ZxdgExportedV1(Interface):
    """An exported surface handle

    An xdg_exported object represents an exported reference to a surface. The
    exported surface may be referenced as long as the xdg_exported object not
    destroyed. Destroying the xdg_exported invalidates any relationship the
    importer may have established using xdg_imported.
    """

    name = "zxdg_exported_v1"
    version = 1


class ZxdgExporterV1(Interface):
    """Interface for exporting surfaces

    A global interface used for exporting surfaces that can later be imported
    using xdg_importer.
    """

    name = "zxdg_exporter_v1"
    version = 1


class ZxdgImportedV1(Interface):
    """An imported surface handle

    An xdg_imported object represents an imported reference to surface exported
    by some client. A client can use this interface to manipulate relationships
    between its own surfaces and the imported surface.
    """

    name = "zxdg_imported_v1"
    version = 1


class ZxdgImporterV1(Interface):
    """Interface for importing surfaces

    A global interface used for importing surfaces exported by xdg_exporter.
    With this interface, a client can create a reference to a surface of
    another client.
    """

    name = "zxdg_importer_v1"
    version = 1


class ZxdgExportedV1Resource(Resource[ZxdgExportedV1]):
    interface = ZxdgExportedV1

    @ZxdgExportedV1.event(
        Argument(ArgumentType.String),
    )
    def handle(self, handle: str) -> None:
        """The exported surface handle

        The handle event contains the unique handle of this exported surface
        reference. It may be shared with any client, which then can use it to
        import the surface by calling xdg_importer.import. A handle may be used
        to import the surface multiple times.

        :param handle:
            the exported surface handle
        :type handle:
            `ArgumentType.String`
        """
        self._post_event(0, handle)


class ZxdgExporterV1Resource(Resource[ZxdgExporterV1]):
    interface = ZxdgExporterV1


class ZxdgImportedV1Resource(Resource[ZxdgImportedV1]):
    interface = ZxdgImportedV1

    @ZxdgImportedV1.event()
    def destroyed(self) -> None:
        """The imported surface handle has been destroyed

        The imported surface handle has been destroyed and any relationship set
        up has been invalidated. This may happen for various reasons, for
        example if the exported surface or the exported surface handle has been
        destroyed, if the handle used for importing was invalid.
        """
        self._post_event(0)


class ZxdgImporterV1Resource(Resource[ZxdgImporterV1]):
    interface = ZxdgImporterV1


class ZxdgExportedV1Proxy(Proxy[ZxdgExportedV1]):
    interface = ZxdgExportedV1

    @ZxdgExportedV1.request()
    def destroy(self) -> None:
        """Unexport the exported surface

        Revoke the previously exported surface. This invalidates any
        relationship the importer may have set up using the xdg_imported
        created given the handle sent via xdg_exported.handle.
        """
        self._marshal(0)
        self._destroy()


class ZxdgExporterV1Proxy(Proxy[ZxdgExporterV1]):
    interface = ZxdgExporterV1

    @ZxdgExporterV1.request()
    def destroy(self) -> None:
        """Destroy the xdg_exporter object

        Notify the compositor that the xdg_exporter object will no longer be
        used.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgExporterV1.request(
        Argument(ArgumentType.NewId, interface=ZxdgExportedV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def export(self, surface: WlSurfaceProxy) -> ZxdgExportedV1Proxy:
        """Export a surface

        The export request exports the passed surface so that it can later be
        imported via xdg_importer. When called, a new xdg_exported object will
        be created and xdg_exported.handle will be sent immediately. See the
        corresponding interface and event for details.

        A surface may be exported multiple times, and each exported handle may
        be used to create an xdg_imported multiple times. Only
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` surfaces may be
        exported.

        :param surface:
            the surface to export
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.xdg_foreign_unstable_v1.ZxdgExportedV1`
            -- the new xdg_exported object
        """
        id = self._marshal_constructor(1, ZxdgExportedV1, surface)
        return id


class ZxdgImportedV1Proxy(Proxy[ZxdgImportedV1]):
    interface = ZxdgImportedV1

    @ZxdgImportedV1.request()
    def destroy(self) -> None:
        """Destroy the xdg_imported object

        Notify the compositor that it will no longer use the xdg_imported
        object. Any relationship that may have been set up will at this point
        be invalidated.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgImportedV1.request(
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def set_parent_of(self, surface: WlSurfaceProxy) -> None:
        """Set as the parent of some surface

        Set the imported surface as the parent of some surface of the client.
        The passed surface must be a toplevel
        :class:`~pywayland.protocol.xdg_shell.XdgSurface`. Calling this
        function sets up a surface to surface relation with the same stacking
        and positioning semantics as :func:`XdgSurface.set_parent()
        <pywayland.protocol.xdg_shell.XdgSurface.set_parent>`.

        :param surface:
            the child surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._marshal(1, surface)


class ZxdgImporterV1Proxy(Proxy[ZxdgImporterV1]):
    interface = ZxdgImporterV1

    @ZxdgImporterV1.request()
    def destroy(self) -> None:
        """Destroy the xdg_importer object

        Notify the compositor that the xdg_importer object will no longer be
        used.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgImporterV1.request(
        Argument(ArgumentType.NewId, interface=ZxdgImportedV1),
        Argument(ArgumentType.String),
    )
    def import_(self, handle: str) -> ZxdgImportedV1Proxy:
        """Import a surface

        The import request imports a surface from any client given a handle
        retrieved by exporting said surface using xdg_exporter.export. When
        called, a new xdg_imported object will be created. This new object
        represents the imported surface, and the importing client can
        manipulate its relationship using it. See xdg_imported for details.

        :param handle:
            the exported surface handle
        :type handle:
            `ArgumentType.String`
        :returns:
            :class:`~pywayland.protocol.xdg_foreign_unstable_v1.ZxdgImportedV1`
            -- the new xdg_imported object
        """
        id = self._marshal_constructor(1, ZxdgImportedV1, handle)
        return id


class ZxdgExportedV1Global(Global[ZxdgExportedV1]):
    interface = ZxdgExportedV1


class ZxdgExporterV1Global(Global[ZxdgExporterV1]):
    interface = ZxdgExporterV1


class ZxdgImportedV1Global(Global[ZxdgImportedV1]):
    interface = ZxdgImportedV1


class ZxdgImporterV1Global(Global[ZxdgImporterV1]):
    interface = ZxdgImporterV1


ZxdgExportedV1._gen_c()
ZxdgExportedV1.proxy_class = ZxdgExportedV1Proxy
ZxdgExportedV1.resource_class = ZxdgExportedV1Resource
ZxdgExportedV1.global_class = ZxdgExportedV1Global


ZxdgExporterV1._gen_c()
ZxdgExporterV1.proxy_class = ZxdgExporterV1Proxy
ZxdgExporterV1.resource_class = ZxdgExporterV1Resource
ZxdgExporterV1.global_class = ZxdgExporterV1Global


ZxdgImportedV1._gen_c()
ZxdgImportedV1.proxy_class = ZxdgImportedV1Proxy
ZxdgImportedV1.resource_class = ZxdgImportedV1Resource
ZxdgImportedV1.global_class = ZxdgImportedV1Global


ZxdgImporterV1._gen_c()
ZxdgImporterV1.proxy_class = ZxdgImporterV1Proxy
ZxdgImporterV1.resource_class = ZxdgImporterV1Resource
ZxdgImporterV1.global_class = ZxdgImporterV1Global
