# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 Simon Ser
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .xdg_shell import XdgToplevel
from .xdg_shell import XdgToplevelProxy


class ZxdgDecorationManagerV1(Interface):
    """Window decoration manager

    This interface allows a compositor to announce support for server-side
    decorations.

    A window decoration is a set of window controls as deemed appropriate by
    the party managing them, such as user interface components used to move,
    resize and change a window's state.

    A client can use this protocol to request being decorated by a supporting
    compositor.

    If compositor and client do not negotiate the use of a server-side
    decoration using this protocol, clients continue to self-decorate as they
    see fit.

    Warning! The protocol described in this file is experimental and backward
    incompatible changes may be made. Backward compatible changes may be added
    together with the corresponding interface version bump. Backward
    incompatible changes are done by bumping the version number in the protocol
    and interface names and resetting the interface version. Once the protocol
    is to be declared stable, the 'z' prefix and the version number in the
    protocol and interface names are removed and the interface version number
    is reset.
    """

    name = "zxdg_decoration_manager_v1"
    version = 2


class ZxdgToplevelDecorationV1(Interface):
    """Decoration object for a toplevel surface

    The decoration object allows the compositor to toggle server-side window
    decorations for a toplevel surface. The client can request to switch to
    another mode.

    The xdg_toplevel_decoration object must be destroyed before its
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel`.
    """

    name = "zxdg_toplevel_decoration_v1"
    version = 2

    class error(enum.IntEnum):
        unconfigured_buffer = 0
        already_constructed = 1
        orphaned = 2
        invalid_mode = 3

    class mode(enum.IntEnum):
        client_side = 1
        server_side = 2


class ZxdgDecorationManagerV1Resource(Resource[ZxdgDecorationManagerV1]):
    interface = ZxdgDecorationManagerV1


class ZxdgToplevelDecorationV1Resource(Resource[ZxdgToplevelDecorationV1]):
    interface = ZxdgToplevelDecorationV1

    @ZxdgToplevelDecorationV1.event(
        Argument(ArgumentType.Uint),
    )
    def configure(self, mode: int) -> None:
        """Notify a decoration mode change

        The configure event configures the effective decoration mode. The
        configured state should not be applied immediately. Clients must send
        an ack_configure in response to this event. See
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` and
        :func:`XdgSurface.ack_configure()
        <pywayland.protocol.xdg_shell.XdgSurface.ack_configure>` for details.

        A configure event can be sent at any time. The specified mode must be
        obeyed by the client.

        :param mode:
            the decoration mode
        :type mode:
            `ArgumentType.Uint`
        """
        self._post_event(0, mode)


class ZxdgDecorationManagerV1Proxy(Proxy[ZxdgDecorationManagerV1]):
    interface = ZxdgDecorationManagerV1

    @ZxdgDecorationManagerV1.request()
    def destroy(self) -> None:
        """Destroy the decoration manager object

        Destroy the decoration manager. This doesn't destroy objects created
        with the manager.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgDecorationManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZxdgToplevelDecorationV1),
        Argument(ArgumentType.Object, interface=XdgToplevel),
    )
    def get_toplevel_decoration(self, toplevel: XdgToplevelProxy) -> ZxdgToplevelDecorationV1Proxy:
        """Create a new toplevel decoration object

        Create a new decoration object associated with the given toplevel.

        For objects of version 1, creating an xdg_toplevel_decoration from an
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` which has a buffer
        attached or committed is a client error, and any attempts by a client
        to attach or manipulate a buffer prior to the first
        xdg_toplevel_decoration.configure event must also be treated as errors.

        For objects of version 2 or newer, creating an xdg_toplevel_decoration
        from an :class:`~pywayland.protocol.xdg_shell.XdgToplevel` which has a
        buffer attached or committed is allowed. The initial decoration mode of
        the surface if a buffer is already attached depends on whether a
        xdg_toplevel_decoration object has been associated with the surface or
        not prior to this request.

        If an xdg_toplevel_decoration was associated with the surface, then
        destroyed without a surface commit, the previous decoration mode is
        retained.

        If no xdg_toplevel_decoration was associated with the surface prior to
        this request, or if a surface commit has been performed after a
        previous xdg_toplevel_decoration object associated with the surface was
        destroyed, the decoration mode is assumed to be client-side.

        :param toplevel:
        :type toplevel:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        :returns:
            :class:`~pywayland.protocol.xdg_decoration_unstable_v1.ZxdgToplevelDecorationV1`
        """
        id = self._marshal_constructor(1, ZxdgToplevelDecorationV1, toplevel)
        return id


class ZxdgToplevelDecorationV1Proxy(Proxy[ZxdgToplevelDecorationV1]):
    interface = ZxdgToplevelDecorationV1

    @ZxdgToplevelDecorationV1.request()
    def destroy(self) -> None:
        """Destroy the decoration object

        Switch back to a mode without any server-side decorations at the next
        commit, unless a new xdg_toplevel_decoration is created for the surface
        first.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgToplevelDecorationV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_mode(self, mode: int) -> None:
        """Set the decoration mode

        Set the toplevel surface decoration mode. This informs the compositor
        that the client prefers the provided decoration mode.

        After requesting a decoration mode, the compositor will respond by
        emitting an :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event. The client
        should then update its content, drawing it without decorations if the
        received mode is server-side decorations. The client must also
        acknowledge the configure when committing the new content (see
        :func:`XdgSurface.ack_configure()
        <pywayland.protocol.xdg_shell.XdgSurface.ack_configure>`).

        The compositor can decide not to use the client's mode and enforce a
        different mode instead.

        Clients whose decoration mode depend on the
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` state may send a
        set_mode request in response to an :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event and wait for
        the next :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event to prevent
        unwanted state. Such clients are responsible for preventing configure
        loops and must make sure not to send multiple successive set_mode
        requests with the same decoration mode.

        If an invalid mode is supplied by the client, the invalid_mode protocol
        error is raised by the compositor.

        :param mode:
            the decoration mode
        :type mode:
            `ArgumentType.Uint`
        """
        self._marshal(1, mode)

    @ZxdgToplevelDecorationV1.request()
    def unset_mode(self) -> None:
        """Unset the decoration mode

        Unset the toplevel surface decoration mode. This informs the compositor
        that the client doesn't prefer a particular decoration mode.

        This request has the same semantics as set_mode.
        """
        self._marshal(2)


class ZxdgDecorationManagerV1Global(Global[ZxdgDecorationManagerV1]):
    interface = ZxdgDecorationManagerV1


class ZxdgToplevelDecorationV1Global(Global[ZxdgToplevelDecorationV1]):
    interface = ZxdgToplevelDecorationV1


ZxdgDecorationManagerV1._gen_c()
ZxdgDecorationManagerV1.proxy_class = ZxdgDecorationManagerV1Proxy
ZxdgDecorationManagerV1.resource_class = ZxdgDecorationManagerV1Resource
ZxdgDecorationManagerV1.global_class = ZxdgDecorationManagerV1Global


ZxdgToplevelDecorationV1._gen_c()
ZxdgToplevelDecorationV1.proxy_class = ZxdgToplevelDecorationV1Proxy
ZxdgToplevelDecorationV1.resource_class = ZxdgToplevelDecorationV1Resource
ZxdgToplevelDecorationV1.global_class = ZxdgToplevelDecorationV1Global
