# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
# Copyright © 2015-2017 Samsung Electronics Co., Ltd
# Copyright © 2015-2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlOutput
from .wayland import WlOutputProxy
from .wayland import WlSeat
from .wayland import WlSeatProxy
from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class XdgPopup(Interface):
    """Short-lived, popup surfaces for menus

    A popup surface is a short-lived, temporary surface. It can be used to
    implement for example menus, popovers, tooltips and other similar user
    interface concepts.

    A popup can be made to take an explicit grab. See :func:`XdgPopup.grab()`
    for details.

    When the popup is dismissed, a popup_done event will be sent out, and at
    the same time the surface will be unmapped. See the
    :func:`XdgPopup.popup_done()` event for details.

    Explicitly destroying the :class:`XdgPopup` object will also dismiss the
    popup and unmap the surface. Clients that want to dismiss the popup when
    another surface of their own is clicked should dismiss the popup using the
    destroy request.

    A newly created :class:`XdgPopup` will be stacked on top of all previously
    created :class:`XdgPopup` surfaces associated with the same
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel`.

    The parent of an :class:`XdgPopup` must be mapped (see the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` description) before the
    :class:`XdgPopup` itself.

    The client must call :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` on the corresponding
    :class:`~pywayland.protocol.wayland.WlSurface` for the :class:`XdgPopup`
    state to take effect.
    """

    name = "xdg_popup"
    version = 7

    class error(enum.IntEnum):
        invalid_grab = 0


class XdgPositioner(Interface):
    """Child surface positioner

    The :class:`XdgPositioner` provides a collection of rules for the placement
    of a child surface relative to a parent surface. Rules can be defined to
    ensure the child surface remains within the visible area's borders, and to
    specify how the child surface changes its position, such as sliding along
    an axis, or flipping around a rectangle. These positioner-created rules are
    constrained by the requirement that a child surface must intersect with or
    be at least partially adjacent to its parent surface.

    See the various requests for details about possible rules.

    At the time of the request, the compositor makes a copy of the rules
    specified by the :class:`XdgPositioner`. Thus, after the request is
    complete the :class:`XdgPositioner` object can be destroyed or reused;
    further changes to the object will have no effect on previous usages.

    For an :class:`XdgPositioner` object to be considered complete, it must
    have a non-zero size set by set_size, and a non-zero anchor rectangle set
    by set_anchor_rect. Passing an incomplete :class:`XdgPositioner` object
    when positioning a surface raises an invalid_positioner error.
    """

    name = "xdg_positioner"
    version = 7

    class error(enum.IntEnum):
        invalid_input = 0

    class anchor(enum.IntEnum):
        none = 0
        top = 1
        bottom = 2
        left = 3
        right = 4
        top_left = 5
        bottom_left = 6
        top_right = 7
        bottom_right = 8

    class gravity(enum.IntEnum):
        none = 0
        top = 1
        bottom = 2
        left = 3
        right = 4
        top_left = 5
        bottom_left = 6
        top_right = 7
        bottom_right = 8

    class constraint_adjustment(enum.IntFlag):
        none = 0
        slide_x = 1
        slide_y = 2
        flip_x = 4
        flip_y = 8
        resize_x = 16
        resize_y = 32


class XdgSurface(Interface):
    """Desktop user interface surface base interface

    An interface that may be implemented by a
    :class:`~pywayland.protocol.wayland.WlSurface`, for implementations that
    provide a desktop-style user interface.

    It provides a base set of functionality required to construct user
    interface elements requiring management by the compositor, such as toplevel
    windows, menus, etc. The types of functionality are split into
    :class:`XdgSurface` roles.

    Creating an :class:`XdgSurface` does not set the role for a
    :class:`~pywayland.protocol.wayland.WlSurface`. In order to map an
    :class:`XdgSurface`, the client must create a role-specific object using,
    e.g., get_toplevel, get_popup. The
    :class:`~pywayland.protocol.wayland.WlSurface` for any given
    :class:`XdgSurface` can have at most one role, and may not be assigned any
    role not based on :class:`XdgSurface`.

    A role must be assigned before any other requests are made to the
    :class:`XdgSurface` object.

    The client must call :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` on the corresponding
    :class:`~pywayland.protocol.wayland.WlSurface` for the :class:`XdgSurface`
    state to take effect.

    Creating an :class:`XdgSurface` from a
    :class:`~pywayland.protocol.wayland.WlSurface` which has a buffer attached
    or committed is a client error, and any attempts by a client to attach or
    manipulate a buffer prior to the first :func:`XdgSurface.configure()` call
    must also be treated as errors.

    After creating a role-specific object and setting it up (e.g. by sending
    the title, app ID, size constraints, parent, etc), the client must perform
    an initial commit without any buffer attached. The compositor will reply
    with initial :class:`~pywayland.protocol.wayland.WlSurface` state such as
    :func:`WlSurface.preferred_buffer_scale()
    <pywayland.protocol.wayland.WlSurface.preferred_buffer_scale>` followed by
    an :func:`XdgSurface.configure()` event. The client must acknowledge it and
    is then allowed to attach a buffer to map the surface.

    Mapping an :class:`XdgSurface`-based role surface is defined as making it
    possible for the surface to be shown by the compositor. Note that a mapped
    surface is not guaranteed to be visible once it is mapped.

    For an :class:`XdgSurface` to be mapped by the compositor, the following
    conditions must be met: (1) the client has assigned an
    :class:`XdgSurface`-based role to the surface (2) the client has set and
    committed the :class:`XdgSurface` state and the     role-dependent state to
    the surface (3) the client has committed a buffer to the surface

    A newly-unmapped surface is considered to have met condition (1) out of the
    3 required conditions for mapping a surface if its role surface has not
    been destroyed, i.e. the client must perform the initial commit again
    before attaching a buffer.
    """

    name = "xdg_surface"
    version = 7

    class error(enum.IntEnum):
        not_constructed = 1
        already_constructed = 2
        unconfigured_buffer = 3
        invalid_serial = 4
        invalid_size = 5
        defunct_role_object = 6


class XdgToplevel(Interface):
    """Toplevel surface

    This interface defines an :class:`~pywayland.protocol.xdg_shell.XdgSurface`
    role which allows a surface to, among other things, set window-like
    properties such as maximize, fullscreen, and minimize, set application-
    specific metadata like title and id, and well as trigger user interactive
    operations such as interactive resize and move.

    An :class:`XdgToplevel` by default is responsible for providing the full
    intended visual representation of the toplevel, which depending on the
    window state, may mean things like a title bar, window controls and drop
    shadow.

    Unmapping an :class:`XdgToplevel` means that the surface cannot be shown by
    the compositor until it is explicitly mapped again. All active operations
    (e.g., move, resize) are canceled and all attributes (e.g. title, state,
    stacking, ...) are discarded for an :class:`XdgToplevel` surface when it is
    unmapped. The :class:`XdgToplevel` returns to the state it had right after
    :func:`XdgSurface.get_toplevel()
    <pywayland.protocol.xdg_shell.XdgSurface.get_toplevel>`. The client can re-
    map the toplevel by performing a commit without any buffer attached,
    waiting for a configure event and handling it as usual (see
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` description).

    Attaching a null buffer to a toplevel unmaps the surface.
    """

    name = "xdg_toplevel"
    version = 7

    class error(enum.IntEnum):
        invalid_resize_edge = 0
        invalid_parent = 1
        invalid_size = 2

    class resize_edge(enum.IntEnum):
        none = 0
        top = 1
        bottom = 2
        left = 4
        top_left = 5
        bottom_left = 6
        right = 8
        top_right = 9
        bottom_right = 10

    class state(enum.IntEnum):
        maximized = 1
        fullscreen = 2
        resizing = 3
        activated = 4
        tiled_left = 5
        tiled_right = 6
        tiled_top = 7
        tiled_bottom = 8
        suspended = 9
        constrained_left = 10
        constrained_right = 11
        constrained_top = 12
        constrained_bottom = 13

    class wm_capabilities(enum.IntEnum):
        window_menu = 1
        maximize = 2
        fullscreen = 3
        minimize = 4


class XdgWmBase(Interface):
    """Create desktop-style surfaces

    The :class:`XdgWmBase` interface is exposed as a global object enabling
    clients to turn their wl_surfaces into windows in a desktop environment. It
    defines the basic functionality needed for clients and the compositor to
    create windows that can be dragged, resized, maximized, etc, as well as
    creating transient windows such as popup menus.
    """

    name = "xdg_wm_base"
    version = 7

    class error(enum.IntEnum):
        role = 0
        defunct_surfaces = 1
        not_the_topmost_popup = 2
        invalid_popup_parent = 3
        invalid_surface_state = 4
        invalid_positioner = 5
        unresponsive = 6


class XdgPopupResource(Resource[XdgPopup]):
    interface = XdgPopup

    @XdgPopup.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def configure(self, x: int, y: int, width: int, height: int) -> None:
        """Configure the popup surface

        This event asks the popup surface to configure itself given the
        configuration. The configured state should not be applied immediately.
        See :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` for details.

        The x and y arguments represent the position the popup was placed at
        given the :class:`~pywayland.protocol.xdg_shell.XdgPositioner` rule,
        relative to the upper left corner of the window geometry of the parent
        surface.

        For version 2 or older, the configure event for an :class:`XdgPopup` is
        only ever sent once for the initial configuration. Starting with
        version 3, it may be sent again if the popup is setup with an
        :class:`~pywayland.protocol.xdg_shell.XdgPositioner` with set_reactive
        requested, or in response to :func:`XdgPopup.reposition()` requests.

        :param x:
            x position relative to parent surface window geometry
        :type x:
            `ArgumentType.Int`
        :param y:
            y position relative to parent surface window geometry
        :type y:
            `ArgumentType.Int`
        :param width:
            window geometry width
        :type width:
            `ArgumentType.Int`
        :param height:
            window geometry height
        :type height:
            `ArgumentType.Int`
        """
        self._post_event(0, x, y, width, height)

    @XdgPopup.event()
    def popup_done(self) -> None:
        """Popup interaction is done

        The popup_done event is sent out when a popup is dismissed by the
        compositor. The client should destroy the :class:`XdgPopup` object at
        this point.
        """
        self._post_event(1)

    @XdgPopup.event(
        Argument(ArgumentType.Uint),
        version=3,
    )
    def repositioned(self, token: int) -> None:
        """Signal the completion of a repositioned request

        The repositioned event is sent as part of a popup configuration
        sequence, together with :func:`XdgPopup.configure()` and lastly
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` to notify the
        completion of a reposition request.

        The repositioned event is to notify about the completion of a
        :func:`XdgPopup.reposition()` request. The token argument is the token
        passed in the :func:`XdgPopup.reposition()` request.

        Immediately after this event is emitted, :func:`XdgPopup.configure()`
        and :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` will be sent with
        the updated size and position, as well as a new configure serial.

        The client should optionally update the content of the popup, but must
        acknowledge the new popup configuration for the new position to take
        effect. See :func:`XdgSurface.ack_configure()
        <pywayland.protocol.xdg_shell.XdgSurface.ack_configure>` for details.

        :param token:
            reposition request token
        :type token:
            `ArgumentType.Uint`
        """
        self._post_event(2, token)


class XdgPositionerResource(Resource[XdgPositioner]):
    interface = XdgPositioner


class XdgSurfaceResource(Resource[XdgSurface]):
    interface = XdgSurface

    @XdgSurface.event(
        Argument(ArgumentType.Uint),
    )
    def configure(self, serial: int) -> None:
        """Suggest a surface change

        The configure event marks the end of a configure sequence. A configure
        sequence is a set of one or more events configuring the state of the
        :class:`XdgSurface`, including the final :func:`XdgSurface.configure()`
        event.

        Where applicable, :class:`XdgSurface` surface roles will during a
        configure sequence extend this event as a latched state sent as events
        before the :func:`XdgSurface.configure()` event. Such events should be
        considered to make up a set of atomically applied configuration states,
        where the :func:`XdgSurface.configure()` commits the accumulated state.

        Clients should arrange their surface for the new states, and then send
        an ack_configure request with the serial sent in this configure event
        at some point before committing the new surface.

        If the client receives multiple configure events before it can respond
        to one, it is free to discard all but the last event it received.

        :param serial:
            serial of the configure event
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(0, serial)


class XdgToplevelResource(Resource[XdgToplevel]):
    interface = XdgToplevel

    @XdgToplevel.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Array),
    )
    def configure(self, width: int, height: int, states: list) -> None:
        """Suggest a surface change

        This configure event asks the client to resize its toplevel surface or
        to change its state. The configured state should not be applied
        immediately. See :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` for details.

        The width and height arguments specify a hint to the window about how
        its surface should be resized in window geometry coordinates. See
        set_window_geometry.

        If the width or height arguments are zero, it means the client should
        decide its own window dimension. This may happen when the compositor
        needs to configure the state of the surface but doesn't have any
        information about any previous or expected dimension.

        The states listed in the event specify how the width/height arguments
        should be interpreted, and possibly how it should be drawn.

        The states are sent as an array of 32-bit unsigned integers in native
        endianness. State values are defined in the state enum.

        Clients must send an ack_configure in response to this event. See
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` and
        :func:`XdgSurface.ack_configure()
        <pywayland.protocol.xdg_shell.XdgSurface.ack_configure>` for details.

        :param width:
            suggested width of window
        :type width:
            `ArgumentType.Int`
        :param height:
            suggested height of window
        :type height:
            `ArgumentType.Int`
        :param states:
            suggested states of the window
        :type states:
            `ArgumentType.Array`
        """
        self._post_event(0, width, height, states)

    @XdgToplevel.event()
    def close(self) -> None:
        """Surface wants to be closed

        The close event is sent by the compositor when the user wants the
        surface to be closed. This should be equivalent to the user clicking
        the close button in client-side decorations, if your application has
        any.

        This is only a request that the user intends to close the window. The
        client may choose to ignore this request, or show a dialog to ask the
        user to save their data, etc.
        """
        self._post_event(1)

    @XdgToplevel.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        version=4,
    )
    def configure_bounds(self, width: int, height: int) -> None:
        """Recommended window geometry bounds

        The configure_bounds event may be sent prior to a
        :func:`XdgToplevel.configure()` event to communicate the bounds a
        window geometry size is recommended to constrain to.

        The passed width and height are in surface coordinate space. If width
        and height are 0, it means bounds is unknown and equivalent to as if no
        configure_bounds event was ever sent for this surface.

        The bounds can for example correspond to the size of a monitor
        excluding any panels or other shell components, so that a surface isn't
        created in a way that it cannot fit.

        The bounds may change at any point, and in such a case, a new
        :func:`XdgToplevel.configure_bounds()` will be sent, followed by
        :func:`XdgToplevel.configure()` and :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>`.

        :param width:
            suggested maximum width of surface
        :type width:
            `ArgumentType.Int`
        :param height:
            suggested maximum height of surface
        :type height:
            `ArgumentType.Int`
        """
        self._post_event(2, width, height)

    @XdgToplevel.event(
        Argument(ArgumentType.Array),
        version=5,
    )
    def wm_capabilities(self, capabilities: list) -> None:
        """Compositor capabilities

        This event advertises the capabilities supported by the compositor. If
        a capability isn't supported, clients should hide or disable the UI
        elements that expose this functionality. For instance, if the
        compositor doesn't advertise support for minimized toplevels, a button
        triggering the set_minimized request should not be displayed.

        The compositor will ignore requests it doesn't support. For instance, a
        compositor which doesn't advertise support for minimized will ignore
        set_minimized requests.

        Compositors must send this event once before the first
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event. When the
        capabilities change, compositors must send this event again and then
        send an :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event.

        The configured state should not be applied immediately. See
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` for details.

        The capabilities are sent as an array of 32-bit unsigned integers in
        native endianness. Capability values are defined in the wm_capabilities
        enum.

        :param capabilities:
            array of 32-bit capabilities
        :type capabilities:
            `ArgumentType.Array`
        """
        self._post_event(3, capabilities)


class XdgWmBaseResource(Resource[XdgWmBase]):
    interface = XdgWmBase

    @XdgWmBase.event(
        Argument(ArgumentType.Uint),
    )
    def ping(self, serial: int) -> None:
        """Check if the client is alive

        The ping event asks the client if it's still alive. Pass the serial
        specified in the event back to the compositor by sending a "pong"
        request back with the specified serial. See :func:`XdgWmBase.pong()`.

        Compositors can use this to determine if the client is still alive.
        It's unspecified what will happen if the client doesn't respond to the
        ping request, or in what timeframe. Clients should try to respond in a
        reasonable amount of time. The “unresponsive” error is provided for
        compositors that wish to disconnect unresponsive clients.

        A compositor is free to ping in any way it wants, but a client must
        always respond to any :class:`XdgWmBase` object it created.

        :param serial:
            pass this to the pong request
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(0, serial)


class XdgPopupProxy(Proxy[XdgPopup]):
    interface = XdgPopup

    @XdgPopup.request()
    def destroy(self) -> None:
        """Remove :class:`XdgPopup` interface

        This destroys the popup. Explicitly destroying the :class:`XdgPopup`
        object will also dismiss the popup, and unmap the surface.

        If this :class:`XdgPopup` is not the "topmost" popup, the
        :func:`XdgWmBase.not_the_topmost_popup()
        <pywayland.protocol.xdg_shell.XdgWmBase.not_the_topmost_popup>`
        protocol error will be sent.
        """
        self._marshal(0)
        self._destroy()

    @XdgPopup.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
    )
    def grab(self, seat: WlSeatProxy, serial: int) -> None:
        """Make the popup take an explicit grab

        This request makes the created popup take an explicit grab. An explicit
        grab will be dismissed when the user dismisses the popup, or when the
        client destroys the :class:`XdgPopup`. This can be done by the user
        clicking outside the surface, using the keyboard, or even locking the
        screen through closing the lid or a timeout.

        If the compositor denies the grab, the popup will be immediately
        dismissed.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event. The serial number of
        the event should be passed as 'serial'.

        The parent of a grabbing popup must either be an
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` surface or another
        :class:`XdgPopup` with an explicit grab. If the parent is another
        :class:`XdgPopup` it means that the popups are nested, with this popup
        now being the topmost popup.

        Nested popups must be destroyed in the reverse order they were created
        in, e.g. the only popup you are allowed to destroy at all times is the
        topmost one.

        When compositors choose to dismiss a popup, they may dismiss every
        nested grabbing popup as well. When a compositor dismisses popups, it
        will follow the same dismissing order as required from the client.

        If the topmost grabbing popup is destroyed, the grab will be returned
        to the parent of the popup, if that parent previously had an explicit
        grab.

        If the parent is a grabbing popup which has already been dismissed,
        this popup will be immediately dismissed. If the parent is a popup that
        did not take an explicit grab, an error will be raised.

        During a popup grab, the client owning the grab will receive pointer
        and touch events for all their surfaces as normal (similar to an
        "owner-events" grab in X11 parlance), while the top most grabbing popup
        will always have keyboard focus.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(1, seat, serial)

    @XdgPopup.request(
        Argument(ArgumentType.Object, interface=XdgPositioner),
        Argument(ArgumentType.Uint),
        version=3,
    )
    def reposition(self, positioner: XdgPositionerProxy, token: int) -> None:
        """Recalculate the popup's location

        Reposition an already-mapped popup. The popup will be placed given the
        details in the passed
        :class:`~pywayland.protocol.xdg_shell.XdgPositioner` object, and a
        :func:`XdgPopup.repositioned()` followed by
        :func:`XdgPopup.configure()` and :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` will be emitted in
        response. Any parameters set by the previous positioner will be
        discarded.

        The passed token will be sent in the corresponding
        :func:`XdgPopup.repositioned()` event. The new popup position will not
        take effect until the corresponding configure event is acknowledged by
        the client. See :func:`XdgPopup.repositioned()` for details. The token
        itself is opaque, and has no other special meaning.

        If multiple reposition requests are sent, the compositor may skip all
        but the last one.

        If the popup is repositioned in response to a configure event for its
        parent, the client should send an
        :func:`XdgPositioner.set_parent_configure()
        <pywayland.protocol.xdg_shell.XdgPositioner.set_parent_configure>` and
        possibly an :func:`XdgPositioner.set_parent_size()
        <pywayland.protocol.xdg_shell.XdgPositioner.set_parent_size>` request
        to allow the compositor to properly constrain the popup.

        If the popup is repositioned together with a parent that is being
        resized, but not in response to a configure event, the client should
        send an :func:`XdgPositioner.set_parent_size()
        <pywayland.protocol.xdg_shell.XdgPositioner.set_parent_size>` request.

        :param positioner:
        :type positioner:
            :class:`~pywayland.protocol.xdg_shell.XdgPositioner`
        :param token:
            reposition request token
        :type token:
            `ArgumentType.Uint`
        """
        self._marshal(2, positioner, token)


class XdgPositionerProxy(Proxy[XdgPositioner]):
    interface = XdgPositioner

    @XdgPositioner.request()
    def destroy(self) -> None:
        """Destroy the :class:`XdgPositioner` object

        Notify the compositor that the :class:`XdgPositioner` will no longer be
        used.
        """
        self._marshal(0)
        self._destroy()

    @XdgPositioner.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_size(self, width: int, height: int) -> None:
        """Set the size of the to-be positioned rectangle

        Set the size of the surface that is to be positioned with the
        positioner object. The size is in surface-local coordinates and
        corresponds to the window geometry. See
        :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>`.

        If a zero or negative size is set the invalid_input error is raised.

        :param width:
            width of positioned rectangle
        :type width:
            `ArgumentType.Int`
        :param height:
            height of positioned rectangle
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(1, width, height)

    @XdgPositioner.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_anchor_rect(self, x: int, y: int, width: int, height: int) -> None:
        """Set the anchor rectangle within the parent surface

        Specify the anchor rectangle within the parent surface that the child
        surface will be placed relative to. The rectangle is relative to the
        window geometry as defined by :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>` of the
        parent surface.

        When the :class:`XdgPositioner` object is used to position a child
        surface, the anchor rectangle may not extend outside the window
        geometry of the positioned child's parent surface.

        If a negative size is set the invalid_input error is raised.

        :param x:
            x position of anchor rectangle
        :type x:
            `ArgumentType.Int`
        :param y:
            y position of anchor rectangle
        :type y:
            `ArgumentType.Int`
        :param width:
            width of anchor rectangle
        :type width:
            `ArgumentType.Int`
        :param height:
            height of anchor rectangle
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(2, x, y, width, height)

    @XdgPositioner.request(
        Argument(ArgumentType.Uint),
    )
    def set_anchor(self, anchor: int) -> None:
        """Set anchor rectangle anchor

        Defines the anchor point for the anchor rectangle. The specified anchor
        is used to derive an anchor point that the child surface will be
        positioned relative to. If a corner anchor is set (e.g. 'top_left' or
        'bottom_right'), the anchor point will be at the specified corner;
        otherwise, the derived anchor point will be centered on the specified
        edge, or in the center of the anchor rectangle if no edge is specified.

        :param anchor:
            anchor point
        :type anchor:
            `ArgumentType.Uint`
        """
        self._marshal(3, anchor)

    @XdgPositioner.request(
        Argument(ArgumentType.Uint),
    )
    def set_gravity(self, gravity: int) -> None:
        """Set child surface gravity

        Defines in what direction a surface should be positioned, relative to
        the anchor point of the parent surface. If a corner gravity is
        specified (e.g. 'bottom_right' or 'top_left'), then the child surface
        will be placed towards the specified gravity; otherwise, the child
        surface will be centered over the anchor point on any axis that had no
        gravity specified. If the gravity is not in the ‘gravity’ enum, an
        invalid_input error is raised.

        :param gravity:
            gravity direction
        :type gravity:
            `ArgumentType.Uint`
        """
        self._marshal(4, gravity)

    @XdgPositioner.request(
        Argument(ArgumentType.Uint),
    )
    def set_constraint_adjustment(self, constraint_adjustment: int) -> None:
        """Set the adjustment to be done when constrained

        Specify how the window should be positioned if the originally intended
        position caused the surface to be constrained, meaning at least
        partially outside positioning boundaries set by the compositor. The
        adjustment is set by constructing a bitmask describing the adjustment
        to be made when the surface is constrained on that axis.

        If no bit for one axis is set, the compositor will assume that the
        child surface should not change its position on that axis when
        constrained.

        If more than one bit for one axis is set, the order of how adjustments
        are applied is specified in the corresponding adjustment descriptions.

        The default adjustment is none.

        :param constraint_adjustment:
            bit mask of constraint adjustments
        :type constraint_adjustment:
            `ArgumentType.Uint`
        """
        self._marshal(5, constraint_adjustment)

    @XdgPositioner.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_offset(self, x: int, y: int) -> None:
        """Set surface position offset

        Specify the surface position offset relative to the position of the
        anchor on the anchor rectangle and the anchor on the surface. For
        example if the anchor of the anchor rectangle is at (x, y), the surface
        has the gravity bottom|right, and the offset is (ox, oy), the
        calculated surface position will be (x + ox, y + oy). The offset
        position of the surface is the one used for constraint testing. See
        set_constraint_adjustment.

        An example use case is placing a popup menu on top of a user interface
        element, while aligning the user interface element of the parent
        surface with some user interface element placed somewhere in the popup
        surface.

        :param x:
            surface position x offset
        :type x:
            `ArgumentType.Int`
        :param y:
            surface position y offset
        :type y:
            `ArgumentType.Int`
        """
        self._marshal(6, x, y)

    @XdgPositioner.request(version=3)
    def set_reactive(self) -> None:
        """Continuously reconstrain the surface

        When set reactive, the surface is reconstrained if the conditions used
        for constraining changed, e.g. the parent window moved.

        If the conditions changed and the popup was reconstrained, an
        :func:`XdgPopup.configure()
        <pywayland.protocol.xdg_shell.XdgPopup.configure>` event is sent with
        updated geometry, followed by an :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event.
        """
        self._marshal(7)

    @XdgPositioner.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        version=3,
    )
    def set_parent_size(self, parent_width: int, parent_height: int) -> None:
        """Set parent size

        Set the parent window geometry the compositor should use when
        positioning the popup. The compositor may use this information to
        determine the future state the popup should be constrained using. If
        this doesn't match the dimension of the parent the popup is eventually
        positioned against, the behavior is undefined.

        The arguments are given in the surface-local coordinate space.

        :param parent_width:
            future window geometry width of parent
        :type parent_width:
            `ArgumentType.Int`
        :param parent_height:
            future window geometry height of parent
        :type parent_height:
            `ArgumentType.Int`
        """
        self._marshal(8, parent_width, parent_height)

    @XdgPositioner.request(
        Argument(ArgumentType.Uint),
        version=3,
    )
    def set_parent_configure(self, serial: int) -> None:
        """Set parent configure this is a response to

        Set the serial of an :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event this
        positioner will be used in response to. The compositor may use this
        information together with set_parent_size to determine what future
        state the popup should be constrained using.

        :param serial:
            serial of parent configure event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(9, serial)


class XdgSurfaceProxy(Proxy[XdgSurface]):
    interface = XdgSurface

    @XdgSurface.request()
    def destroy(self) -> None:
        """Destroy the :class:`XdgSurface`

        Destroy the :class:`XdgSurface` object. An :class:`XdgSurface` must
        only be destroyed after its role object has been destroyed, otherwise a
        defunct_role_object error is raised.
        """
        self._marshal(0)
        self._destroy()

    @XdgSurface.request(
        Argument(ArgumentType.NewId, interface=XdgToplevel),
    )
    def get_toplevel(self) -> XdgToplevelProxy:
        """Assign the :class:`~pywayland.protocol.xdg_shell.XdgToplevel` surface role

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        object for the given :class:`XdgSurface` and gives the associated
        :class:`~pywayland.protocol.wayland.WlSurface` the
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` role.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` for more details
        about what an :class:`~pywayland.protocol.xdg_shell.XdgToplevel` is and
        how it is used.

        :returns:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        """
        id = self._marshal_constructor(1, XdgToplevel)
        return id

    @XdgSurface.request(
        Argument(ArgumentType.NewId, interface=XdgPopup),
        Argument(ArgumentType.Object, interface=XdgSurface, nullable=True),
        Argument(ArgumentType.Object, interface=XdgPositioner),
    )
    def get_popup(self, parent: XdgSurfaceProxy | None, positioner: XdgPositionerProxy) -> XdgPopupProxy:
        """Assign the :class:`~pywayland.protocol.xdg_shell.XdgPopup` surface role

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgPopup` object
        for the given :class:`XdgSurface` and gives the associated
        :class:`~pywayland.protocol.wayland.WlSurface` the
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` role.

        If null is passed as a parent, a parent surface must be specified using
        some other protocol, before committing the initial state.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` for more details about
        what an :class:`~pywayland.protocol.xdg_shell.XdgPopup` is and how it
        is used.

        :param parent:
            parent surface for this popup
        :type parent:
            :class:`XdgSurface` or `None`
        :param positioner:
            positioner for this popup
        :type positioner:
            :class:`~pywayland.protocol.xdg_shell.XdgPositioner`
        :returns:
            :class:`~pywayland.protocol.xdg_shell.XdgPopup`
        """
        id = self._marshal_constructor(2, XdgPopup, parent, positioner)
        return id

    @XdgSurface.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_window_geometry(self, x: int, y: int, width: int, height: int) -> None:
        """Set the new window geometry

        The window geometry of a surface is its "visible bounds" from the
        user's perspective. Client-side decorations often have invisible
        portions like drop-shadows which should be ignored for the purposes of
        aligning, placing and constraining windows. Note that in some
        situations, compositors may clip rendering to the window geometry, so
        the client should avoid putting functional elements outside of it.

        The window geometry is double-buffered state, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        When maintaining a position, the compositor should treat the (x, y)
        coordinate of the window geometry as the top left corner of the window.
        A client changing the (x, y) window geometry coordinate should in
        general not alter the position of the window.

        Once the window geometry of the surface is set, it is not possible to
        unset it, and it will remain the same until set_window_geometry is
        called again, even if a new subsurface or buffer is attached.

        If never set, the value is the full bounds of the surface, including
        any subsurfaces. This updates dynamically on every commit. This unset
        is meant for extremely simple clients.

        The arguments are given in the surface-local coordinate space of the
        :class:`~pywayland.protocol.wayland.WlSurface` associated with this
        :class:`XdgSurface`, and may extend outside of the
        :class:`~pywayland.protocol.wayland.WlSurface` itself to mark parts of
        the subsurface tree as part of the window geometry.

        When applied, the effective window geometry will be the set window
        geometry clamped to the bounding rectangle of the combined geometry of
        the surface of the :class:`XdgSurface` and the associated subsurfaces.

        The effective geometry will not be recalculated unless a new call to
        set_window_geometry is done and the new pending surface state is
        subsequently applied.

        The width and height of the effective window geometry must be greater
        than zero. Setting an invalid size will raise an invalid_size error.

        :param x:
            x coordinate of the top-left corner of the window inside this
            surface
        :type x:
            `ArgumentType.Int`
        :param y:
            y coordinate of the top-left corner of the window inside this
            surface
        :type y:
            `ArgumentType.Int`
        :param width:
            width of the window
        :type width:
            `ArgumentType.Int`
        :param height:
            height of the window
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(3, x, y, width, height)

    @XdgSurface.request(
        Argument(ArgumentType.Uint),
    )
    def ack_configure(self, serial: int) -> None:
        """Ack a configure event

        When a configure event is received, if a client commits the surface in
        response to the configure event, then the client must make an
        ack_configure request sometime before the commit request, passing along
        the serial of the configure event.

        For instance, for toplevel surfaces the compositor might use this
        information to move a surface to the top left only when the client has
        drawn itself for the maximized or fullscreen state.

        If the client receives multiple configure events before it can respond
        to one, it only has to ack the last configure event. Acking a configure
        event that was never sent raises an invalid_serial error.

        A client is not required to commit immediately after sending an
        ack_configure request - it may even ack_configure several times before
        its next surface commit.

        A client may send multiple ack_configure requests before committing,
        but only the last request sent before a commit indicates which
        configure event the client really is responding to.

        Sending an ack_configure request consumes the serial number sent with
        the request, as well as serial numbers sent by all configure events
        sent on this :class:`XdgSurface` prior to the configure event
        referenced by the committed serial.

        It is an error to issue multiple ack_configure requests referencing a
        serial from the same configure event, or to issue an ack_configure
        request referencing a serial from a configure event issued before the
        event identified by the last ack_configure request for the same
        :class:`XdgSurface`. Doing so will raise an invalid_serial error.

        :param serial:
            the serial from the configure event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(4, serial)


class XdgToplevelProxy(Proxy[XdgToplevel]):
    interface = XdgToplevel

    @XdgToplevel.request()
    def destroy(self) -> None:
        """Destroy the :class:`XdgToplevel`

        This request destroys the role surface and unmaps the surface; see
        "Unmapping" behavior in interface section for details.
        """
        self._marshal(0)
        self._destroy()

    @XdgToplevel.request(
        Argument(ArgumentType.Object, interface=XdgToplevel, nullable=True),
    )
    def set_parent(self, parent: XdgToplevelProxy | None) -> None:
        """Set the parent of this surface

        Set the "parent" of this surface. This surface should be stacked above
        the parent surface and all other ancestor surfaces.

        Parent surfaces should be set on dialogs, toolboxes, or other
        "auxiliary" surfaces, so that the parent is raised when the dialog is
        raised.

        Setting a null parent for a child surface unsets its parent. Setting a
        null parent for a surface which currently has no parent is a no-op.

        Only mapped surfaces can have child surfaces. Setting a parent which is
        not mapped is equivalent to setting a null parent. If a surface becomes
        unmapped, its children's parent is set to the parent of the now-
        unmapped surface. If the now-unmapped surface has no parent, its
        children's parent is unset. If the now-unmapped surface becomes mapped
        again, its parent-child relationship is not restored.

        The parent toplevel must not be one of the child toplevel's
        descendants, and the parent must be different from the child toplevel,
        otherwise the invalid_parent protocol error is raised.

        :param parent:
            parent surface for this surface
        :type parent:
            :class:`XdgToplevel` or `None`
        """
        self._marshal(1, parent)

    @XdgToplevel.request(
        Argument(ArgumentType.String),
    )
    def set_title(self, title: str) -> None:
        """Set surface title

        Set a short title for the surface.

        This string may be used to identify the surface in a task bar, window
        list, or other user interface elements provided by the compositor.

        The string must be encoded in UTF-8.

        :param title:
            title of the surface
        :type title:
            `ArgumentType.String`
        """
        self._marshal(2, title)

    @XdgToplevel.request(
        Argument(ArgumentType.String),
    )
    def set_app_id(self, app_id: str) -> None:
        """Set application id

        Set an application identifier for the surface.

        The app ID identifies the general class of applications to which the
        surface belongs. The compositor can use this to group multiple surfaces
        together, or to determine how to launch a new application.

        For D-Bus activatable applications, the app ID is used as the D-Bus
        service name.

        The compositor shell will try to group application surfaces together by
        their app ID. As a best practice, it is suggested to select app ID's
        that match the basename of the application's .desktop file. For
        example, "org.freedesktop.FooViewer" where the .desktop file is
        "org.freedesktop.FooViewer.desktop".

        Like other properties, a set_app_id request can be sent after the
        :class:`XdgToplevel` has been mapped to update the property.

        See the desktop-entry specification [0] for more details on application
        identifiers and how they relate to well-known D-Bus names and .desktop
        files.

        [0] https://standards.freedesktop.org/desktop-entry-spec/

        :param app_id:
            application identifier surface belongs to
        :type app_id:
            `ArgumentType.String`
        """
        self._marshal(3, app_id)

    @XdgToplevel.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def show_window_menu(self, seat: WlSeatProxy, serial: int, x: int, y: int) -> None:
        """Show the window menu

        Clients implementing client-side decorations might want to show a
        context menu when right-clicking on the decorations, giving the user a
        menu that they can use to maximize or minimize the window.

        This request asks the compositor to pop up such a window menu at the
        given position, relative to the local surface coordinates of the parent
        surface. There are no guarantees as to what menu items the window menu
        contains, or even if a window menu will be drawn at all.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        :param x:
            the x position to pop up the window menu at
        :type x:
            `ArgumentType.Int`
        :param y:
            the y position to pop up the window menu at
        :type y:
            `ArgumentType.Int`
        """
        self._marshal(4, seat, serial, x, y)

    @XdgToplevel.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
    )
    def move(self, seat: WlSeatProxy, serial: int) -> None:
        """Start an interactive move

        Start an interactive, user-driven move of the surface.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event. The passed serial is
        used to determine the type of interactive move (touch, pointer, etc).

        The server may ignore move requests depending on the state of the
        surface (e.g. fullscreen or maximized), or if the passed serial is no
        longer valid.

        If triggered, the surface will lose the focus of the device
        (:class:`~pywayland.protocol.wayland.WlPointer`,
        :class:`~pywayland.protocol.wayland.WlTouch`, etc) used for the move.
        It is up to the compositor to visually indicate that the move is taking
        place, such as updating a pointer cursor, during the move. There is no
        guarantee that the device focus will return when the move is completed.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(5, seat, serial)

    @XdgToplevel.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def resize(self, seat: WlSeatProxy, serial: int, edges: int) -> None:
        """Start an interactive resize

        Start a user-driven, interactive resize of the surface.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event. The passed serial is
        used to determine the type of interactive resize (touch, pointer, etc).

        The server may ignore resize requests depending on the state of the
        surface (e.g. fullscreen or maximized).

        If triggered, the client will receive configure events with the
        "resize" state enum value and the expected sizes. See the "resize" enum
        value for more details about what is required. The client must also
        acknowledge configure events using "ack_configure". After the resize is
        completed, the client will receive another "configure" event without
        the resize state.

        If triggered, the surface also will lose the focus of the device
        (:class:`~pywayland.protocol.wayland.WlPointer`,
        :class:`~pywayland.protocol.wayland.WlTouch`, etc) used for the resize.
        It is up to the compositor to visually indicate that the resize is
        taking place, such as updating a pointer cursor, during the resize.
        There is no guarantee that the device focus will return when the resize
        is completed.

        The edges parameter specifies how the surface should be resized, and is
        one of the values of the resize_edge enum. Values not matching a
        variant of the enum will cause the invalid_resize_edge protocol error.
        The compositor may use this information to update the surface position
        for example when dragging the top left corner. The compositor may also
        use this information to adapt its behavior, e.g. choose an appropriate
        cursor image.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        :param edges:
            which edge or corner is being dragged
        :type edges:
            `ArgumentType.Uint`
        """
        self._marshal(6, seat, serial, edges)

    @XdgToplevel.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_max_size(self, width: int, height: int) -> None:
        """Set the maximum size

        Set a maximum size for the window.

        The client can specify a maximum size so that the compositor does not
        try to configure the window beyond this size.

        The width and height arguments are in window geometry coordinates. See
        :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>`.

        Values set in this way are double-buffered, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        The compositor can use this information to allow or disallow different
        states like maximize or fullscreen and draw accurate animations.

        Similarly, a tiling window manager may use this information to place
        and resize client windows in a more effective way.

        The client should not rely on the compositor to obey the maximum size.
        The compositor may decide to ignore the values set by the client and
        request a larger size.

        If never set, or a value of zero in the request, means that the client
        has no expected maximum size in the given dimension. As a result, a
        client wishing to reset the maximum size to an unspecified state can
        use zero for width and height in the request.

        Requesting a maximum size to be smaller than the minimum size of a
        surface is illegal and will result in an invalid_size error.

        The width and height must be greater than or equal to zero. Using
        strictly negative values for width or height will result in an
        invalid_size error.

        :param width:
            maximum width of the window
        :type width:
            `ArgumentType.Int`
        :param height:
            maximum height of the window
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(7, width, height)

    @XdgToplevel.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_min_size(self, width: int, height: int) -> None:
        """Set the minimum size

        Set a minimum size for the window.

        The client can specify a minimum size so that the compositor does not
        try to configure the window below this size.

        The width and height arguments are in window geometry coordinates. See
        :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>`.

        Values set in this way are double-buffered, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        The compositor can use this information to allow or disallow different
        states like maximize or fullscreen and draw accurate animations.

        Similarly, a tiling window manager may use this information to place
        and resize client windows in a more effective way.

        The client should not rely on the compositor to obey the minimum size.
        The compositor may decide to ignore the values set by the client and
        request a smaller size.

        If never set, or a value of zero in the request, means that the client
        has no expected minimum size in the given dimension. As a result, a
        client wishing to reset the minimum size to an unspecified state can
        use zero for width and height in the request.

        Requesting a minimum size to be larger than the maximum size of a
        surface is illegal and will result in an invalid_size error.

        The width and height must be greater than or equal to zero. Using
        strictly negative values for width and height will result in an
        invalid_size error.

        :param width:
            minimum width of the window
        :type width:
            `ArgumentType.Int`
        :param height:
            minimum height of the window
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(8, width, height)

    @XdgToplevel.request()
    def set_maximized(self) -> None:
        """Maximize the window

        Maximize the surface.

        After requesting that the surface should be maximized, the compositor
        will respond by emitting a configure event. Whether this configure
        actually sets the window maximized is subject to compositor policies.
        The client must then update its content, drawing in the configured
        state. The client must also acknowledge the configure when committing
        the new content (see ack_configure).

        It is up to the compositor to decide how and where to maximize the
        surface, for example which output and what region of the screen should
        be used.

        If the surface was already maximized, the compositor will still emit a
        configure event with the "maximized" state.

        If the surface is in a fullscreen state, this request has no direct
        effect. It may alter the state the surface is returned to when
        unmaximized unless overridden by the compositor.
        """
        self._marshal(9)

    @XdgToplevel.request()
    def unset_maximized(self) -> None:
        """Unmaximize the window

        Unmaximize the surface.

        After requesting that the surface should be unmaximized, the compositor
        will respond by emitting a configure event. Whether this actually un-
        maximizes the window is subject to compositor policies. If available
        and applicable, the compositor will include the window geometry
        dimensions the window had prior to being maximized in the configure
        event. The client must then update its content, drawing it in the
        configured state. The client must also acknowledge the configure when
        committing the new content (see ack_configure).

        It is up to the compositor to position the surface after it was
        unmaximized; usually the position the surface had before maximizing, if
        applicable.

        If the surface was already not maximized, the compositor will still
        emit a configure event without the "maximized" state.

        If the surface is in a fullscreen state, this request has no direct
        effect. It may alter the state the surface is returned to when
        unmaximized unless overridden by the compositor.
        """
        self._marshal(10)

    @XdgToplevel.request(
        Argument(ArgumentType.Object, interface=WlOutput, nullable=True),
    )
    def set_fullscreen(self, output: WlOutputProxy | None) -> None:
        """Set the window as fullscreen on an output

        Make the surface fullscreen.

        After requesting that the surface should be fullscreened, the
        compositor will respond by emitting a configure event. Whether the
        client is actually put into a fullscreen state is subject to compositor
        policies. The client must also acknowledge the configure when
        committing the new content (see ack_configure).

        The output passed by the request indicates the client's preference as
        to which display it should be set fullscreen on. If this value is NULL,
        it's up to the compositor to choose which display will be used to map
        this surface.

        If the surface doesn't cover the whole output, the compositor will
        position the surface in the center of the output and compensate with
        border fill covering the rest of the output. The content of the border
        fill is undefined, but should be assumed to be in some way that
        attempts to blend into the surrounding area (e.g. solid black).

        If the fullscreened surface is not opaque, the compositor must make
        sure that other screen content not part of the same surface tree (made
        up of subsurfaces, popups or similarly coupled surfaces) are not
        visible below the fullscreened surface.

        :param output:
            preferred output to place surface on
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput` or `None`
        """
        self._marshal(11, output)

    @XdgToplevel.request()
    def unset_fullscreen(self) -> None:
        """Unset the window as fullscreen

        Make the surface no longer fullscreen.

        After requesting that the surface should be unfullscreened, the
        compositor will respond by emitting a configure event. Whether this
        actually removes the fullscreen state of the client is subject to
        compositor policies.

        Making a surface unfullscreen sets states for the surface based on the
        following: * the state(s) it may have had before becoming fullscreen *
        any state(s) decided by the compositor * any state(s) requested by the
        client while the surface was fullscreen

        The compositor may include the previous window geometry dimensions in
        the configure event, if applicable.

        The client must also acknowledge the configure when committing the new
        content (see ack_configure).
        """
        self._marshal(12)

    @XdgToplevel.request()
    def set_minimized(self) -> None:
        """Set the window as minimized

        Request that the compositor minimize your surface. There is no way to
        know if the surface is currently minimized, nor is there any way to
        unset minimization on this surface.

        If you are looking to throttle redrawing when minimized, please instead
        use the :func:`WlSurface.frame()
        <pywayland.protocol.wayland.WlSurface.frame>` event for this, as this
        will also work with live previews on windows in Alt-Tab, Expose or
        similar compositor features.
        """
        self._marshal(13)


class XdgWmBaseProxy(Proxy[XdgWmBase]):
    interface = XdgWmBase

    @XdgWmBase.request()
    def destroy(self) -> None:
        """Destroy :class:`XdgWmBase`

        Destroy this :class:`XdgWmBase` object.

        Destroying a bound :class:`XdgWmBase` object while there are surfaces
        still alive created by this :class:`XdgWmBase` object instance is
        illegal and will result in a defunct_surfaces error.
        """
        self._marshal(0)
        self._destroy()

    @XdgWmBase.request(
        Argument(ArgumentType.NewId, interface=XdgPositioner),
    )
    def create_positioner(self) -> XdgPositionerProxy:
        """Create a positioner object

        Create a positioner object. A positioner object is used to position
        surfaces relative to some parent surface. See the interface description
        and :func:`XdgSurface.get_popup()
        <pywayland.protocol.xdg_shell.XdgSurface.get_popup>` for details.

        :returns:
            :class:`~pywayland.protocol.xdg_shell.XdgPositioner`
        """
        id = self._marshal_constructor(1, XdgPositioner)
        return id

    @XdgWmBase.request(
        Argument(ArgumentType.NewId, interface=XdgSurface),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_xdg_surface(self, surface: WlSurfaceProxy) -> XdgSurfaceProxy:
        """Create a shell surface from a surface

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgSurface` for
        the given surface. An :class:`~pywayland.protocol.xdg_shell.XdgSurface`
        is used as basis to define a role to a given surface, such as
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` or
        :class:`~pywayland.protocol.xdg_shell.XdgPopup`. It also manages
        functionality shared between
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` based surface roles.

        While :class:`~pywayland.protocol.xdg_shell.XdgSurface` itself is not a
        role, the corresponding surface may only be assigned a role extending
        :class:`~pywayland.protocol.xdg_shell.XdgSurface`, such as
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` or
        :class:`~pywayland.protocol.xdg_shell.XdgPopup`. It is illegal to
        create an :class:`~pywayland.protocol.xdg_shell.XdgSurface` for a
        :class:`~pywayland.protocol.wayland.WlSurface` which already has
        anassigned role and this will result in a role error.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` for more details
        about what an :class:`~pywayland.protocol.xdg_shell.XdgSurface` is and
        how it is used.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.xdg_shell.XdgSurface`
        """
        id = self._marshal_constructor(2, XdgSurface, surface)
        return id

    @XdgWmBase.request(
        Argument(ArgumentType.Uint),
    )
    def pong(self, serial: int) -> None:
        """Respond to a ping event

        A client must respond to a ping event with a pong request or the client
        may be deemed unresponsive. See :func:`XdgWmBase.ping()` and
        :func:`XdgWmBase.error()`.unresponsive.

        :param serial:
            serial of the ping event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(3, serial)


class XdgPopupGlobal(Global[XdgPopup]):
    interface = XdgPopup


class XdgPositionerGlobal(Global[XdgPositioner]):
    interface = XdgPositioner


class XdgSurfaceGlobal(Global[XdgSurface]):
    interface = XdgSurface


class XdgToplevelGlobal(Global[XdgToplevel]):
    interface = XdgToplevel


class XdgWmBaseGlobal(Global[XdgWmBase]):
    interface = XdgWmBase


XdgPopup._gen_c()
XdgPopup.proxy_class = XdgPopupProxy
XdgPopup.resource_class = XdgPopupResource
XdgPopup.global_class = XdgPopupGlobal


XdgPositioner._gen_c()
XdgPositioner.proxy_class = XdgPositionerProxy
XdgPositioner.resource_class = XdgPositionerResource
XdgPositioner.global_class = XdgPositionerGlobal


XdgSurface._gen_c()
XdgSurface.proxy_class = XdgSurfaceProxy
XdgSurface.resource_class = XdgSurfaceResource
XdgSurface.global_class = XdgSurfaceGlobal


XdgToplevel._gen_c()
XdgToplevel.proxy_class = XdgToplevelProxy
XdgToplevel.resource_class = XdgToplevelResource
XdgToplevel.global_class = XdgToplevelGlobal


XdgWmBase._gen_c()
XdgWmBase.proxy_class = XdgWmBaseProxy
XdgWmBase.resource_class = XdgWmBaseResource
XdgWmBase.global_class = XdgWmBaseGlobal
