# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlOutput
from .wayland import WlOutputProxy
from .wayland import WlSeat
from .wayland import WlSeatProxy
from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class ZxdgPopupV6(Interface):
    """Short-lived, popup surfaces for menus

    A popup surface is a short-lived, temporary surface. It can be used to
    implement for example menus, popovers, tooltips and other similar user
    interface concepts.

    A popup can be made to take an explicit grab. See :func:`XdgPopup.grab()
    <pywayland.protocol.xdg_shell.XdgPopup.grab>` for details.

    When the popup is dismissed, a popup_done event will be sent out, and at
    the same time the surface will be unmapped. See the
    :func:`XdgPopup.popup_done()
    <pywayland.protocol.xdg_shell.XdgPopup.popup_done>` event for details.

    Explicitly destroying the :class:`~pywayland.protocol.xdg_shell.XdgPopup`
    object will also dismiss the popup and unmap the surface. Clients that want
    to dismiss the popup when another surface of their own is clicked should
    dismiss the popup using the destroy request.

    The parent surface must have either the
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` or
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` surface role.

    A newly created :class:`~pywayland.protocol.xdg_shell.XdgPopup` will be
    stacked on top of all previously created
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` surfaces associated with
    the same :class:`~pywayland.protocol.xdg_shell.XdgToplevel`.

    The parent of an :class:`~pywayland.protocol.xdg_shell.XdgPopup` must be
    mapped (see the :class:`~pywayland.protocol.xdg_shell.XdgSurface`
    description) before the :class:`~pywayland.protocol.xdg_shell.XdgPopup`
    itself.

    The x and y arguments passed when creating the popup object specify where
    the top left of the popup should be placed, relative to the local surface
    coordinates of the parent surface. See :func:`XdgSurface.get_popup()
    <pywayland.protocol.xdg_shell.XdgSurface.get_popup>`. An
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` must intersect with or be
    at least partially adjacent to its parent surface.

    The client must call :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` on the corresponding
    :class:`~pywayland.protocol.wayland.WlSurface` for the
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` state to take effect.
    """

    name = "zxdg_popup_v6"
    version = 1

    class error(enum.IntEnum):
        invalid_grab = 0


class ZxdgPositionerV6(Interface):
    """Child surface positioner

    The :class:`~pywayland.protocol.xdg_shell.XdgPositioner` provides a
    collection of rules for the placement of a child surface relative to a
    parent surface. Rules can be defined to ensure the child surface remains
    within the visible area's borders, and to specify how the child surface
    changes its position, such as sliding along an axis, or flipping around a
    rectangle. These positioner-created rules are constrained by the
    requirement that a child surface must intersect with or be at least
    partially adjacent to its parent surface.

    See the various requests for details about possible rules.

    At the time of the request, the compositor makes a copy of the rules
    specified by the :class:`~pywayland.protocol.xdg_shell.XdgPositioner`.
    Thus, after the request is complete the
    :class:`~pywayland.protocol.xdg_shell.XdgPositioner` object can be
    destroyed or reused; further changes to the object will have no effect on
    previous usages.

    For an :class:`~pywayland.protocol.xdg_shell.XdgPositioner` object to be
    considered complete, it must have a non-zero size set by set_size, and a
    non-zero anchor rectangle set by set_anchor_rect. Passing an incomplete
    :class:`~pywayland.protocol.xdg_shell.XdgPositioner` object when
    positioning a surface raises an error.
    """

    name = "zxdg_positioner_v6"
    version = 1

    class error(enum.IntEnum):
        invalid_input = 0

    class anchor(enum.IntFlag):
        none = 0
        top = 1
        bottom = 2
        left = 4
        right = 8

    class gravity(enum.IntFlag):
        none = 0
        top = 1
        bottom = 2
        left = 4
        right = 8

    class constraint_adjustment(enum.IntFlag):
        none = 0
        slide_x = 1
        slide_y = 2
        flip_x = 4
        flip_y = 8
        resize_x = 16
        resize_y = 32


class ZxdgShellV6(Interface):
    """Create desktop-style surfaces

    :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell` allows clients
    to turn a :class:`~pywayland.protocol.wayland.WlSurface` into a "real
    window" which can be dragged, resized, stacked, and moved around by the
    user. Everything about this interface is suited towards traditional desktop
    environments.
    """

    name = "zxdg_shell_v6"
    version = 1

    class error(enum.IntEnum):
        role = 0
        defunct_surfaces = 1
        not_the_topmost_popup = 2
        invalid_popup_parent = 3
        invalid_surface_state = 4
        invalid_positioner = 5


class ZxdgSurfaceV6(Interface):
    """Desktop user interface surface base interface

    An interface that may be implemented by a
    :class:`~pywayland.protocol.wayland.WlSurface`, for implementations that
    provide a desktop-style user interface.

    It provides a base set of functionality required to construct user
    interface elements requiring management by the compositor, such as toplevel
    windows, menus, etc. The types of functionality are split into
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` roles.

    Creating an :class:`~pywayland.protocol.xdg_shell.XdgSurface` does not set
    the role for a :class:`~pywayland.protocol.wayland.WlSurface`. In order to
    map an :class:`~pywayland.protocol.xdg_shell.XdgSurface`, the client must
    create a role-specific object using, e.g., get_toplevel, get_popup. The
    :class:`~pywayland.protocol.wayland.WlSurface` for any given
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` can have at most one
    role, and may not be assigned any role not based on
    :class:`~pywayland.protocol.xdg_shell.XdgSurface`.

    A role must be assigned before any other requests are made to the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` object.

    The client must call :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` on the corresponding
    :class:`~pywayland.protocol.wayland.WlSurface` for the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` state to take effect.

    Creating an :class:`~pywayland.protocol.xdg_shell.XdgSurface` from a
    :class:`~pywayland.protocol.wayland.WlSurface` which has a buffer attached
    or committed is a client error, and any attempts by a client to attach or
    manipulate a buffer prior to the first :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface.configure>` call must also be
    treated as errors.

    For a surface to be mapped by the compositor, the following conditions must
    be met: (1) the client has assigned an
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` based role to the
    surface, (2) the client has set and committed the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` state and the role
    dependent state to the surface and (3) the client has committed a buffer to
    the surface.
    """

    name = "zxdg_surface_v6"
    version = 1

    class error(enum.IntEnum):
        not_constructed = 1
        already_constructed = 2
        unconfigured_buffer = 3


class ZxdgToplevelV6(Interface):
    """Toplevel surface

    This interface defines an :class:`~pywayland.protocol.xdg_shell.XdgSurface`
    role which allows a surface to, among other things, set window-like
    properties such as maximize, fullscreen, and minimize, set application-
    specific metadata like title and id, and well as trigger user interactive
    operations such as interactive resize and move.
    """

    name = "zxdg_toplevel_v6"
    version = 1

    class resize_edge(enum.IntEnum):
        none = 0
        top = 1
        bottom = 2
        left = 4
        top_left = 5
        bottom_left = 6
        right = 8
        top_right = 9
        bottom_right = 10

    class state(enum.IntEnum):
        maximized = 1
        fullscreen = 2
        resizing = 3
        activated = 4


class ZxdgPopupV6Resource(Resource[ZxdgPopupV6]):
    interface = ZxdgPopupV6

    @ZxdgPopupV6.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def configure(self, x: int, y: int, width: int, height: int) -> None:
        """Configure the popup surface

        This event asks the popup surface to configure itself given the
        configuration. The configured state should not be applied immediately.
        See :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` for details.

        The x and y arguments represent the position the popup was placed at
        given the :class:`~pywayland.protocol.xdg_shell.XdgPositioner` rule,
        relative to the upper left corner of the window geometry of the parent
        surface.

        :param x:
            x position relative to parent surface window geometry
        :type x:
            `ArgumentType.Int`
        :param y:
            y position relative to parent surface window geometry
        :type y:
            `ArgumentType.Int`
        :param width:
            window geometry width
        :type width:
            `ArgumentType.Int`
        :param height:
            window geometry height
        :type height:
            `ArgumentType.Int`
        """
        self._post_event(0, x, y, width, height)

    @ZxdgPopupV6.event()
    def popup_done(self) -> None:
        """Popup interaction is done

        The popup_done event is sent out when a popup is dismissed by the
        compositor. The client should destroy the
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` object at this point.
        """
        self._post_event(1)


class ZxdgPositionerV6Resource(Resource[ZxdgPositionerV6]):
    interface = ZxdgPositionerV6


class ZxdgShellV6Resource(Resource[ZxdgShellV6]):
    interface = ZxdgShellV6

    @ZxdgShellV6.event(
        Argument(ArgumentType.Uint),
    )
    def ping(self, serial: int) -> None:
        """Check if the client is alive

        The ping event asks the client if it's still alive. Pass the serial
        specified in the event back to the compositor by sending a "pong"
        request back with the specified serial. See :func:`XdgShell.ping()
        <pywayland.protocol.xdg_shell_unstable_v5.XdgShell.ping>`.

        Compositors can use this to determine if the client is still alive.
        It's unspecified what will happen if the client doesn't respond to the
        ping request, or in what timeframe. Clients should try to respond in a
        reasonable amount of time.

        A compositor is free to ping in any way it wants, but a client must
        always respond to any
        :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell` object it
        created.

        :param serial:
            pass this to the pong request
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(0, serial)


class ZxdgSurfaceV6Resource(Resource[ZxdgSurfaceV6]):
    interface = ZxdgSurfaceV6

    @ZxdgSurfaceV6.event(
        Argument(ArgumentType.Uint),
    )
    def configure(self, serial: int) -> None:
        """Suggest a surface change

        The configure event marks the end of a configure sequence. A configure
        sequence is a set of one or more events configuring the state of the
        :class:`~pywayland.protocol.xdg_shell.XdgSurface`, including the final
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event.

        Where applicable, :class:`~pywayland.protocol.xdg_shell.XdgSurface`
        surface roles will during a configure sequence extend this event as a
        latched state sent as events before the :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event. Such events
        should be considered to make up a set of atomically applied
        configuration states, where the :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` commits the
        accumulated state.

        Clients should arrange their surface for the new states, and then send
        an ack_configure request with the serial sent in this configure event
        at some point before committing the new surface.

        If the client receives multiple configure events before it can respond
        to one, it is free to discard all but the last event it received.

        :param serial:
            serial of the configure event
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(0, serial)


class ZxdgToplevelV6Resource(Resource[ZxdgToplevelV6]):
    interface = ZxdgToplevelV6

    @ZxdgToplevelV6.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Array),
    )
    def configure(self, width: int, height: int, states: list) -> None:
        """Suggest a surface change

        This configure event asks the client to resize its toplevel surface or
        to change its state. The configured state should not be applied
        immediately. See :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` for details.

        The width and height arguments specify a hint to the window about how
        its surface should be resized in window geometry coordinates. See
        set_window_geometry.

        If the width or height arguments are zero, it means the client should
        decide its own window dimension. This may happen when the compositor
        needs to configure the state of the surface but doesn't have any
        information about any previous or expected dimension.

        The states listed in the event specify how the width/height arguments
        should be interpreted, and possibly how it should be drawn.

        Clients must send an ack_configure in response to this event. See
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` and
        :func:`XdgSurface.ack_configure()
        <pywayland.protocol.xdg_shell.XdgSurface.ack_configure>` for details.

        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        :param states:
        :type states:
            `ArgumentType.Array`
        """
        self._post_event(0, width, height, states)

    @ZxdgToplevelV6.event()
    def close(self) -> None:
        """Surface wants to be closed

        The close event is sent by the compositor when the user wants the
        surface to be closed. This should be equivalent to the user clicking
        the close button in client-side decorations, if your application has
        any.

        This is only a request that the user intends to close the window. The
        client may choose to ignore this request, or show a dialog to ask the
        user to save their data, etc.
        """
        self._post_event(1)


class ZxdgPopupV6Proxy(Proxy[ZxdgPopupV6]):
    interface = ZxdgPopupV6

    @ZxdgPopupV6.request()
    def destroy(self) -> None:
        """Remove :class:`~pywayland.protocol.xdg_shell.XdgPopup` interface

        This destroys the popup. Explicitly destroying the
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` object will also
        dismiss the popup, and unmap the surface.

        If this :class:`~pywayland.protocol.xdg_shell.XdgPopup` is not the
        "topmost" popup, a protocol error will be sent.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgPopupV6.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
    )
    def grab(self, seat: WlSeatProxy, serial: int) -> None:
        """Make the popup take an explicit grab

        This request makes the created popup take an explicit grab. An explicit
        grab will be dismissed when the user dismisses the popup, or when the
        client destroys the :class:`~pywayland.protocol.xdg_shell.XdgPopup`.
        This can be done by the user clicking outside the surface, using the
        keyboard, or even locking the screen through closing the lid or a
        timeout.

        If the compositor denies the grab, the popup will be immediately
        dismissed.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event. The serial number of
        the event should be passed as 'serial'.

        The parent of a grabbing popup must either be an
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` surface or another
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` with an explicit grab.
        If the parent is another
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` it means that the
        popups are nested, with this popup now being the topmost popup.

        Nested popups must be destroyed in the reverse order they were created
        in, e.g. the only popup you are allowed to destroy at all times is the
        topmost one.

        When compositors choose to dismiss a popup, they may dismiss every
        nested grabbing popup as well. When a compositor dismisses popups, it
        will follow the same dismissing order as required from the client.

        The parent of a grabbing popup must either be another
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` with an active explicit
        grab, or an :class:`~pywayland.protocol.xdg_shell.XdgPopup` or
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel`, if there are no
        explicit grabs already taken.

        If the topmost grabbing popup is destroyed, the grab will be returned
        to the parent of the popup, if that parent previously had an explicit
        grab.

        If the parent is a grabbing popup which has already been dismissed,
        this popup will be immediately dismissed. If the parent is a popup that
        did not take an explicit grab, an error will be raised.

        During a popup grab, the client owning the grab will receive pointer
        and touch events for all their surfaces as normal (similar to an
        "owner-events" grab in X11 parlance), while the top most grabbing popup
        will always have keyboard focus.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(1, seat, serial)


class ZxdgPositionerV6Proxy(Proxy[ZxdgPositionerV6]):
    interface = ZxdgPositionerV6

    @ZxdgPositionerV6.request()
    def destroy(self) -> None:
        """Destroy the :class:`~pywayland.protocol.xdg_shell.XdgPositioner` object

        Notify the compositor that the
        :class:`~pywayland.protocol.xdg_shell.XdgPositioner` will no longer be
        used.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgPositionerV6.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_size(self, width: int, height: int) -> None:
        """Set the size of the to-be positioned rectangle

        Set the size of the surface that is to be positioned with the
        positioner object. The size is in surface-local coordinates and
        corresponds to the window geometry. See
        :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>`.

        If a zero or negative size is set the invalid_input error is raised.

        :param width:
            width of positioned rectangle
        :type width:
            `ArgumentType.Int`
        :param height:
            height of positioned rectangle
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(1, width, height)

    @ZxdgPositionerV6.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_anchor_rect(self, x: int, y: int, width: int, height: int) -> None:
        """Set the anchor rectangle within the parent surface

        Specify the anchor rectangle within the parent surface that the child
        surface will be placed relative to. The rectangle is relative to the
        window geometry as defined by :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>` of the
        parent surface. The rectangle must be at least 1x1 large.

        When the :class:`~pywayland.protocol.xdg_shell.XdgPositioner` object is
        used to position a child surface, the anchor rectangle may not extend
        outside the window geometry of the positioned child's parent surface.

        If a zero or negative size is set the invalid_input error is raised.

        :param x:
            x position of anchor rectangle
        :type x:
            `ArgumentType.Int`
        :param y:
            y position of anchor rectangle
        :type y:
            `ArgumentType.Int`
        :param width:
            width of anchor rectangle
        :type width:
            `ArgumentType.Int`
        :param height:
            height of anchor rectangle
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(2, x, y, width, height)

    @ZxdgPositionerV6.request(
        Argument(ArgumentType.Uint),
    )
    def set_anchor(self, anchor: int) -> None:
        """Set anchor rectangle anchor edges

        Defines a set of edges for the anchor rectangle. These are used to
        derive an anchor point that the child surface will be positioned
        relative to. If two orthogonal edges are specified (e.g. 'top' and
        'left'), then the anchor point will be the intersection of the edges
        (e.g. the top left position of the rectangle); otherwise, the derived
        anchor point will be centered on the specified edge, or in the center
        of the anchor rectangle if no edge is specified.

        If two parallel anchor edges are specified (e.g. 'left' and 'right'),
        the invalid_input error is raised.

        :param anchor:
            bit mask of anchor edges
        :type anchor:
            `ArgumentType.Uint`
        """
        self._marshal(3, anchor)

    @ZxdgPositionerV6.request(
        Argument(ArgumentType.Uint),
    )
    def set_gravity(self, gravity: int) -> None:
        """Set child surface gravity

        Defines in what direction a surface should be positioned, relative to
        the anchor point of the parent surface. If two orthogonal gravities are
        specified (e.g. 'bottom' and 'right'), then the child surface will be
        placed in the specified direction; otherwise, the child surface will be
        centered over the anchor point on any axis that had no gravity
        specified.

        If two parallel gravities are specified (e.g. 'left' and 'right'), the
        invalid_input error is raised.

        :param gravity:
            bit mask of gravity directions
        :type gravity:
            `ArgumentType.Uint`
        """
        self._marshal(4, gravity)

    @ZxdgPositionerV6.request(
        Argument(ArgumentType.Uint),
    )
    def set_constraint_adjustment(self, constraint_adjustment: int) -> None:
        """Set the adjustment to be done when constrained

        Specify how the window should be positioned if the originally intended
        position caused the surface to be constrained, meaning at least
        partially outside positioning boundaries set by the compositor. The
        adjustment is set by constructing a bitmask describing the adjustment
        to be made when the surface is constrained on that axis.

        If no bit for one axis is set, the compositor will assume that the
        child surface should not change its position on that axis when
        constrained.

        If more than one bit for one axis is set, the order of how adjustments
        are applied is specified in the corresponding adjustment descriptions.

        The default adjustment is none.

        :param constraint_adjustment:
            bit mask of constraint adjustments
        :type constraint_adjustment:
            `ArgumentType.Uint`
        """
        self._marshal(5, constraint_adjustment)

    @ZxdgPositionerV6.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_offset(self, x: int, y: int) -> None:
        """Set surface position offset

        Specify the surface position offset relative to the position of the
        anchor on the anchor rectangle and the anchor on the surface. For
        example if the anchor of the anchor rectangle is at (x, y), the surface
        has the gravity bottom|right, and the offset is (ox, oy), the
        calculated surface position will be (x + ox, y + oy). The offset
        position of the surface is the one used for constraint testing. See
        set_constraint_adjustment.

        An example use case is placing a popup menu on top of a user interface
        element, while aligning the user interface element of the parent
        surface with some user interface element placed somewhere in the popup
        surface.

        :param x:
            surface position x offset
        :type x:
            `ArgumentType.Int`
        :param y:
            surface position y offset
        :type y:
            `ArgumentType.Int`
        """
        self._marshal(6, x, y)


class ZxdgShellV6Proxy(Proxy[ZxdgShellV6]):
    interface = ZxdgShellV6

    @ZxdgShellV6.request()
    def destroy(self) -> None:
        """Destroy :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell`

        Destroy this
        :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell` object.

        Destroying a bound
        :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell` object
        while there are surfaces still alive created by this
        :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell` object
        instance is illegal and will result in a protocol error.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgShellV6.request(
        Argument(ArgumentType.NewId, interface=ZxdgPositionerV6),
    )
    def create_positioner(self) -> ZxdgPositionerV6Proxy:
        """Create a positioner object

        Create a positioner object. A positioner object is used to position
        surfaces relative to some parent surface. See the interface description
        and :func:`XdgSurface.get_popup()
        <pywayland.protocol.xdg_shell.XdgSurface.get_popup>` for details.

        :returns:
            :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgPositionerV6`
        """
        id = self._marshal_constructor(1, ZxdgPositionerV6)
        return id

    @ZxdgShellV6.request(
        Argument(ArgumentType.NewId, interface=ZxdgSurfaceV6),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_xdg_surface(self, surface: WlSurfaceProxy) -> ZxdgSurfaceV6Proxy:
        """Create a shell surface from a surface

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgSurface` for
        the given surface. While
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` itself is not a role,
        the corresponding surface may only be assigned a role extending
        :class:`~pywayland.protocol.xdg_shell.XdgSurface`, such as
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` or
        :class:`~pywayland.protocol.xdg_shell.XdgPopup`.

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgSurface` for
        the given surface. An :class:`~pywayland.protocol.xdg_shell.XdgSurface`
        is used as basis to define a role to a given surface, such as
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` or
        :class:`~pywayland.protocol.xdg_shell.XdgPopup`. It also manages
        functionality shared between
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` based surface roles.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` for more details
        about what an :class:`~pywayland.protocol.xdg_shell.XdgSurface` is and
        how it is used.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgSurfaceV6`
        """
        id = self._marshal_constructor(2, ZxdgSurfaceV6, surface)
        return id

    @ZxdgShellV6.request(
        Argument(ArgumentType.Uint),
    )
    def pong(self, serial: int) -> None:
        """Respond to a ping event

        A client must respond to a ping event with a pong request or the client
        may be deemed unresponsive. See :func:`XdgShell.ping()
        <pywayland.protocol.xdg_shell_unstable_v5.XdgShell.ping>`.

        :param serial:
            serial of the ping event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(3, serial)


class ZxdgSurfaceV6Proxy(Proxy[ZxdgSurfaceV6]):
    interface = ZxdgSurfaceV6

    @ZxdgSurfaceV6.request()
    def destroy(self) -> None:
        """Destroy the :class:`~pywayland.protocol.xdg_shell.XdgSurface`

        Destroy the :class:`~pywayland.protocol.xdg_shell.XdgSurface` object.
        An :class:`~pywayland.protocol.xdg_shell.XdgSurface` must only be
        destroyed after its role object has been destroyed. If the role object
        still exists when this request is issued, the
        :func:`ZxdgShellV6.defunct_surfaces()
        <pywayland.protocol.xdg_shell_unstable_v6.ZxdgShellV6.defunct_surfaces>`
        is raised.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgSurfaceV6.request(
        Argument(ArgumentType.NewId, interface=ZxdgToplevelV6),
    )
    def get_toplevel(self) -> ZxdgToplevelV6Proxy:
        """Assign the :class:`~pywayland.protocol.xdg_shell.XdgToplevel` surface role

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        object for the given :class:`~pywayland.protocol.xdg_shell.XdgSurface`
        and gives the associated :class:`~pywayland.protocol.wayland.WlSurface`
        the :class:`~pywayland.protocol.xdg_shell.XdgToplevel` role. If the
        surface already had a role, the :func:`ZxdgShellV6.role()
        <pywayland.protocol.xdg_shell_unstable_v6.ZxdgShellV6.role>` error is
        raised.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` for more details
        about what an :class:`~pywayland.protocol.xdg_shell.XdgToplevel` is and
        how it is used.

        :returns:
            :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgToplevelV6`
        """
        id = self._marshal_constructor(1, ZxdgToplevelV6)
        return id

    @ZxdgSurfaceV6.request(
        Argument(ArgumentType.NewId, interface=ZxdgPopupV6),
        Argument(ArgumentType.Object, interface=ZxdgSurfaceV6),
        Argument(ArgumentType.Object, interface=ZxdgPositionerV6),
    )
    def get_popup(self, parent: ZxdgSurfaceV6Proxy, positioner: ZxdgPositionerV6Proxy) -> ZxdgPopupV6Proxy:
        """Assign the :class:`~pywayland.protocol.xdg_shell.XdgPopup` surface role

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgPopup` object
        for the given :class:`~pywayland.protocol.xdg_shell.XdgSurface` and
        gives the associated :class:`~pywayland.protocol.wayland.WlSurface` the
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` role. If the surface
        already had a role, the :func:`ZxdgShellV6.role()
        <pywayland.protocol.xdg_shell_unstable_v6.ZxdgShellV6.role>` error is
        raised.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` for more details about
        what an :class:`~pywayland.protocol.xdg_shell.XdgPopup` is and how it
        is used.

        :param parent:
        :type parent:
            :class:`ZxdgSurfaceV6`
        :param positioner:
        :type positioner:
            :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgPositionerV6`
        :returns:
            :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgPopupV6`
        """
        id = self._marshal_constructor(2, ZxdgPopupV6, parent, positioner)
        return id

    @ZxdgSurfaceV6.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_window_geometry(self, x: int, y: int, width: int, height: int) -> None:
        """Set the new window geometry

        The window geometry of a surface is its "visible bounds" from the
        user's perspective. Client-side decorations often have invisible
        portions like drop-shadows which should be ignored for the purposes of
        aligning, placing and constraining windows.

        The window geometry is double-buffered state, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        Once the window geometry of the surface is set, it is not possible to
        unset it, and it will remain the same until set_window_geometry is
        called again, even if a new subsurface or buffer is attached.

        If never set, the value is the full bounds of the surface, including
        any subsurfaces. This updates dynamically on every commit. This unset
        is meant for extremely simple clients.

        The arguments are given in the surface-local coordinate space of the
        :class:`~pywayland.protocol.wayland.WlSurface` associated with this
        :class:`~pywayland.protocol.xdg_shell.XdgSurface`.

        The width and height must be greater than zero. Setting an invalid size
        will raise an error. When applied, the effective window geometry will
        be the set window geometry clamped to the bounding rectangle of the
        combined geometry of the surface of the
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` and the associated
        subsurfaces.

        :param x:
        :type x:
            `ArgumentType.Int`
        :param y:
        :type y:
            `ArgumentType.Int`
        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(3, x, y, width, height)

    @ZxdgSurfaceV6.request(
        Argument(ArgumentType.Uint),
    )
    def ack_configure(self, serial: int) -> None:
        """Ack a configure event

        When a configure event is received, if a client commits the surface in
        response to the configure event, then the client must make an
        ack_configure request sometime before the commit request, passing along
        the serial of the configure event.

        For instance, for toplevel surfaces the compositor might use this
        information to move a surface to the top left only when the client has
        drawn itself for the maximized or fullscreen state.

        If the client receives multiple configure events before it can respond
        to one, it only has to ack the last configure event.

        A client is not required to commit immediately after sending an
        ack_configure request - it may even ack_configure several times before
        its next surface commit.

        A client may send multiple ack_configure requests before committing,
        but only the last request sent before a commit indicates which
        configure event the client really is responding to.

        If an invalid serial is used, the
        :func:`ZxdgShellV6.invalid_surface_state()
        <pywayland.protocol.xdg_shell_unstable_v6.ZxdgShellV6.invalid_surface_state>`
        error is raised.

        :param serial:
            the serial from the configure event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(4, serial)


class ZxdgToplevelV6Proxy(Proxy[ZxdgToplevelV6]):
    interface = ZxdgToplevelV6

    @ZxdgToplevelV6.request()
    def destroy(self) -> None:
        """Destroy the :class:`~pywayland.protocol.xdg_shell.XdgToplevel`

        Unmap and destroy the window. The window will be effectively hidden
        from the user's point of view, and all state like maximization,
        fullscreen, and so on, will be lost.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgToplevelV6.request(
        Argument(ArgumentType.Object, interface=ZxdgToplevelV6, nullable=True),
    )
    def set_parent(self, parent: ZxdgToplevelV6Proxy | None) -> None:
        """Set the parent of this surface

        Set the "parent" of this surface. This window should be stacked above a
        parent. The parent surface must be mapped as long as this surface is
        mapped.

        Parent windows should be set on dialogs, toolboxes, or other
        "auxiliary" surfaces, so that the parent is raised when the dialog is
        raised.

        :param parent:
        :type parent:
            :class:`ZxdgToplevelV6` or `None`
        """
        self._marshal(1, parent)

    @ZxdgToplevelV6.request(
        Argument(ArgumentType.String),
    )
    def set_title(self, title: str) -> None:
        """Set surface title

        Set a short title for the surface.

        This string may be used to identify the surface in a task bar, window
        list, or other user interface elements provided by the compositor.

        The string must be encoded in UTF-8.

        :param title:
        :type title:
            `ArgumentType.String`
        """
        self._marshal(2, title)

    @ZxdgToplevelV6.request(
        Argument(ArgumentType.String),
    )
    def set_app_id(self, app_id: str) -> None:
        """Set application id

        Set an application identifier for the surface.

        The app ID identifies the general class of applications to which the
        surface belongs. The compositor can use this to group multiple surfaces
        together, or to determine how to launch a new application.

        For D-Bus activatable applications, the app ID is used as the D-Bus
        service name.

        The compositor shell will try to group application surfaces together by
        their app ID. As a best practice, it is suggested to select app ID's
        that match the basename of the application's .desktop file. For
        example, "org.freedesktop.FooViewer" where the .desktop file is
        "org.freedesktop.FooViewer.desktop".

        See the desktop-entry specification [0] for more details on application
        identifiers and how they relate to well-known D-Bus names and .desktop
        files.

        [0] http://standards.freedesktop.org/desktop-entry-spec/

        :param app_id:
        :type app_id:
            `ArgumentType.String`
        """
        self._marshal(3, app_id)

    @ZxdgToplevelV6.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def show_window_menu(self, seat: WlSeatProxy, serial: int, x: int, y: int) -> None:
        """Show the window menu

        Clients implementing client-side decorations might want to show a
        context menu when right-clicking on the decorations, giving the user a
        menu that they can use to maximize or minimize the window.

        This request asks the compositor to pop up such a window menu at the
        given position, relative to the local surface coordinates of the parent
        surface. There are no guarantees as to what menu items the window menu
        contains.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        :param x:
            the x position to pop up the window menu at
        :type x:
            `ArgumentType.Int`
        :param y:
            the y position to pop up the window menu at
        :type y:
            `ArgumentType.Int`
        """
        self._marshal(4, seat, serial, x, y)

    @ZxdgToplevelV6.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
    )
    def move(self, seat: WlSeatProxy, serial: int) -> None:
        """Start an interactive move

        Start an interactive, user-driven move of the surface.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event. The passed serial is
        used to determine the type of interactive move (touch, pointer, etc).

        The server may ignore move requests depending on the state of the
        surface (e.g. fullscreen or maximized), or if the passed serial is no
        longer valid.

        If triggered, the surface will lose the focus of the device
        (:class:`~pywayland.protocol.wayland.WlPointer`,
        :class:`~pywayland.protocol.wayland.WlTouch`, etc) used for the move.
        It is up to the compositor to visually indicate that the move is taking
        place, such as updating a pointer cursor, during the move. There is no
        guarantee that the device focus will return when the move is completed.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(5, seat, serial)

    @ZxdgToplevelV6.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def resize(self, seat: WlSeatProxy, serial: int, edges: int) -> None:
        """Start an interactive resize

        Start a user-driven, interactive resize of the surface.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event. The passed serial is
        used to determine the type of interactive resize (touch, pointer, etc).

        The server may ignore resize requests depending on the state of the
        surface (e.g. fullscreen or maximized).

        If triggered, the client will receive configure events with the
        "resize" state enum value and the expected sizes. See the "resize" enum
        value for more details about what is required. The client must also
        acknowledge configure events using "ack_configure". After the resize is
        completed, the client will receive another "configure" event without
        the resize state.

        If triggered, the surface also will lose the focus of the device
        (:class:`~pywayland.protocol.wayland.WlPointer`,
        :class:`~pywayland.protocol.wayland.WlTouch`, etc) used for the resize.
        It is up to the compositor to visually indicate that the resize is
        taking place, such as updating a pointer cursor, during the resize.
        There is no guarantee that the device focus will return when the resize
        is completed.

        The edges parameter specifies how the surface should be resized, and is
        one of the values of the resize_edge enum. The compositor may use this
        information to update the surface position for example when dragging
        the top left corner. The compositor may also use this information to
        adapt its behavior, e.g. choose an appropriate cursor image.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        :param edges:
            which edge or corner is being dragged
        :type edges:
            `ArgumentType.Uint`
        """
        self._marshal(6, seat, serial, edges)

    @ZxdgToplevelV6.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_max_size(self, width: int, height: int) -> None:
        """Set the maximum size

        Set a maximum size for the window.

        The client can specify a maximum size so that the compositor does not
        try to configure the window beyond this size.

        The width and height arguments are in window geometry coordinates. See
        :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>`.

        Values set in this way are double-buffered, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        The compositor can use this information to allow or disallow different
        states like maximize or fullscreen and draw accurate animations.

        Similarly, a tiling window manager may use this information to place
        and resize client windows in a more effective way.

        The client should not rely on the compositor to obey the maximum size.
        The compositor may decide to ignore the values set by the client and
        request a larger size.

        If never set, or a value of zero in the request, means that the client
        has no expected maximum size in the given dimension. As a result, a
        client wishing to reset the maximum size to an unspecified state can
        use zero for width and height in the request.

        Requesting a maximum size to be smaller than the minimum size of a
        surface is illegal and will result in a protocol error.

        The width and height must be greater than or equal to zero. Using
        strictly negative values for width and height will result in the
        :func:`ZxdgShellV6.invalid_surface_state()
        <pywayland.protocol.xdg_shell_unstable_v6.ZxdgShellV6.invalid_surface_state>`
        error being raised.

        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(7, width, height)

    @ZxdgToplevelV6.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_min_size(self, width: int, height: int) -> None:
        """Set the minimum size

        Set a minimum size for the window.

        The client can specify a minimum size so that the compositor does not
        try to configure the window below this size.

        The width and height arguments are in window geometry coordinates. See
        :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>`.

        Values set in this way are double-buffered, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        The compositor can use this information to allow or disallow different
        states like maximize or fullscreen and draw accurate animations.

        Similarly, a tiling window manager may use this information to place
        and resize client windows in a more effective way.

        The client should not rely on the compositor to obey the minimum size.
        The compositor may decide to ignore the values set by the client and
        request a smaller size.

        If never set, or a value of zero in the request, means that the client
        has no expected minimum size in the given dimension. As a result, a
        client wishing to reset the minimum size to an unspecified state can
        use zero for width and height in the request.

        Requesting a minimum size to be larger than the maximum size of a
        surface is illegal and will result in a protocol error.

        The width and height must be greater than or equal to zero. Using
        strictly negative values for width and height will result in the
        :func:`ZxdgShellV6.invalid_surface_state()
        <pywayland.protocol.xdg_shell_unstable_v6.ZxdgShellV6.invalid_surface_state>`
        error being raised.

        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(8, width, height)

    @ZxdgToplevelV6.request()
    def set_maximized(self) -> None:
        """Maximize the window

        Maximize the surface.

        After requesting that the surface should be maximized, the compositor
        will respond by emitting a configure event with the "maximized" state
        and the required window geometry. The client should then update its
        content, drawing it in a maximized state, i.e. without shadow or other
        decoration outside of the window geometry. The client must also
        acknowledge the configure when committing the new content (see
        ack_configure).

        It is up to the compositor to decide how and where to maximize the
        surface, for example which output and what region of the screen should
        be used.

        If the surface was already maximized, the compositor will still emit a
        configure event with the "maximized" state.

        Note that unrelated compositor side state changes may cause configure
        events to be emitted at any time, meaning trying to match this request
        to a specific future configure event is futile.
        """
        self._marshal(9)

    @ZxdgToplevelV6.request()
    def unset_maximized(self) -> None:
        """Unmaximize the window

        Unmaximize the surface.

        After requesting that the surface should be unmaximized, the compositor
        will respond by emitting a configure event without the "maximized"
        state. If available, the compositor will include the window geometry
        dimensions the window had prior to being maximized in the configure
        request. The client must then update its content, drawing it in a
        regular state, i.e. potentially with shadow, etc. The client must also
        acknowledge the configure when committing the new content (see
        ack_configure).

        It is up to the compositor to position the surface after it was
        unmaximized; usually the position the surface had before maximizing, if
        applicable.

        If the surface was already not maximized, the compositor will still
        emit a configure event without the "maximized" state.

        Note that unrelated changes in the state of compositor may cause
        configure events to be emitted by the compositor between processing
        this request and emitting corresponding configure event, so trying to
        match the request with the event is futile.
        """
        self._marshal(10)

    @ZxdgToplevelV6.request(
        Argument(ArgumentType.Object, interface=WlOutput, nullable=True),
    )
    def set_fullscreen(self, output: WlOutputProxy | None) -> None:
        """Set the window as fullscreen on a monitor

        Make the surface fullscreen.

        You can specify an output that you would prefer to be fullscreen. If
        this value is NULL, it's up to the compositor to choose which display
        will be used to map this surface.

        If the surface doesn't cover the whole output, the compositor will
        position the surface in the center of the output and compensate with
        black borders filling the rest of the output.

        :param output:
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput` or `None`
        """
        self._marshal(11, output)

    @ZxdgToplevelV6.request()
    def unset_fullscreen(self) -> None:
        """unset_fullscreen
        """
        self._marshal(12)

    @ZxdgToplevelV6.request()
    def set_minimized(self) -> None:
        """Set the window as minimized

        Request that the compositor minimize your surface. There is no way to
        know if the surface is currently minimized, nor is there any way to
        unset minimization on this surface.

        If you are looking to throttle redrawing when minimized, please instead
        use the :func:`WlSurface.frame()
        <pywayland.protocol.wayland.WlSurface.frame>` event for this, as this
        will also work with live previews on windows in Alt-Tab, Expose or
        similar compositor features.
        """
        self._marshal(13)


class ZxdgPopupV6Global(Global[ZxdgPopupV6]):
    interface = ZxdgPopupV6


class ZxdgPositionerV6Global(Global[ZxdgPositionerV6]):
    interface = ZxdgPositionerV6


class ZxdgShellV6Global(Global[ZxdgShellV6]):
    interface = ZxdgShellV6


class ZxdgSurfaceV6Global(Global[ZxdgSurfaceV6]):
    interface = ZxdgSurfaceV6


class ZxdgToplevelV6Global(Global[ZxdgToplevelV6]):
    interface = ZxdgToplevelV6


ZxdgPopupV6._gen_c()
ZxdgPopupV6.proxy_class = ZxdgPopupV6Proxy
ZxdgPopupV6.resource_class = ZxdgPopupV6Resource
ZxdgPopupV6.global_class = ZxdgPopupV6Global


ZxdgPositionerV6._gen_c()
ZxdgPositionerV6.proxy_class = ZxdgPositionerV6Proxy
ZxdgPositionerV6.resource_class = ZxdgPositionerV6Resource
ZxdgPositionerV6.global_class = ZxdgPositionerV6Global


ZxdgShellV6._gen_c()
ZxdgShellV6.proxy_class = ZxdgShellV6Proxy
ZxdgShellV6.resource_class = ZxdgShellV6Resource
ZxdgShellV6.global_class = ZxdgShellV6Global


ZxdgSurfaceV6._gen_c()
ZxdgSurfaceV6.proxy_class = ZxdgSurfaceV6Proxy
ZxdgSurfaceV6.resource_class = ZxdgSurfaceV6Resource
ZxdgSurfaceV6.global_class = ZxdgSurfaceV6Global


ZxdgToplevelV6._gen_c()
ZxdgToplevelV6.proxy_class = ZxdgToplevelV6Proxy
ZxdgToplevelV6.resource_class = ZxdgToplevelV6Resource
ZxdgToplevelV6.global_class = ZxdgToplevelV6Global
